/*
 * Copyright 2007-2010 Claudio Castiglia
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/*
 * Extensible Shell example: Command Module.
 * This module adds the "exec" command to the esh.
 */
#include <sys/types.h>
#include <sys/wait.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include <esh.h>		/* Extensible Shell API */


/**
 * "exec" command.
 */
static void
exec(int fd, char *file)
{
	char *args[2], msg[256];
	int len, st;
	pid_t pid;

	if (*file == '\0') {
		/*
		 * Nothing to execute.
		 */
		return;
	}

	args[0] = file;
	args[1] = NULL;

	if ((pid = fork()) == 0) {
		dup2(fd, STDIN_FILENO);
		dup2(fd, STDOUT_FILENO);
		dup2(fd, STDERR_FILENO);

		if (execve(args[0], args, NULL) < 0) {
			fprintf(stderr, "exec: Can't execute %s: %s\n\r",
			    args[0], strerror(errno));
		}
		exit(-1);
		/* NOTREACHED */
	}

	waitpid(pid, &st, 0);

	len = snprintf(msg, sizeof(msg), "exec: %s terminated with code %d\n\r",
	    args[0], st);
	write(fd, msg, len);
}


/**
 * Add the "exec" command to the esh server.
 */
void
_init()
{
	esh_add("exec", "exec <bin>\n\r\tLaunch the specified file.\n\r", exec);
}


