/*
 * Copyright 2007-2010 Claudio Castiglia
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/*
 * Extensible Shell example: Command Module.
 * This module adds the "show" command to the esh.
 */
#include <stdio.h>
#include <string.h>
#include <unistd.h>

#include <esh.h>		/* Extensible Shell API */


/**
 * "show" command.
 */
static void
show(int fd, char *args)
{
	char buf[1024];
	ssize_t siz;
	FILE *f;

	if (*args == '\0') {
		/*
		 * Nothing to show.
		 */
		return;
	}

	f = fopen(args, "r");
	if (f == NULL) {
		siz = snprintf(buf, sizeof(buf), "Can't open %s: %s\n",
		    args, strerror(errno));
		write(fd, buf, siz);
		return;
	}

	while ((siz = fread(buf, 1, sizeof(buf), f)) != 0) {
		write(fd, buf, siz);
	}

	fclose(f);
}


/**
 * Add the "show" command to the esh server.
 */
void
_init()
{
	esh_add("show", "show <file>\n\r\tShow the content of a file.\n\r",
	    show);
}


