#
# Copyright 2007-2010 Claudio Castiglia
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
# Package Downloader.
# This .mk uses openssl(1) and wget(1).
#
# DWN_PKGNAME=	(NEEDED) Name of the package directory once unpacked
#
# DWN_PKGURL=	(NEEDED) URL (path) of package to download
#
# DWN_PKGTAR=	(NEEDED) Package to download
#
# DWN_PKGDIR=	(OPTIONAL) Directory where to download the package;
#		default is ./
#
# DWN_LOCALPKG=	(OPTIONAL) Full path of local copy of the package;
#		default is ${DWN_PKGDIR}/${DWN_PKGTAR}
#
# DWN_HASH=	(OPTIONAL) File containing the sha1 of the package to download;
#		default is ${DWN_PKGTAR}.sign
#
# DWN_UNTAR=	(OPTIONAL) Unpacker;
#		default is "tar zmxf"
#
# DWN_UNTARDIR=	(OPTIONAL) Directory where to extract the package.
#		It will be created if does not exists.
#		Default is ./
#
# DWN_UNPACKEDIR=
#		(OPTIONAL) Directory where the contents are placed once
#		unpacked. Default is ${DWN_UNTARDIR}/${DWN_PKGNAME}
#
# To generate the ${DWN_HASH} signature file you need the openssl, then:
# make -f download.mk mksum DWN_LOCALPKG=<path> > package.sign
#
DWN_PKGDIR?=	.
DWN_HASH?=	${DWN_PKGTAR}.sign
DWN_LOCALPKG?=	${DWN_PKGDIR}/${DWN_PKGTAR}
DWN_UNPACKEDIR?=${DWN_UNTARDIR}/${DWN_PKGNAME}

DWN_UNTAR?=	tar zmxf
DWN_UNTARDIR?=	.

CLEANDIRFILES+=	${DWN_PKGNAME}

_HASH=		`openssl dgst -sha1 ${DWN_LOCALPKG} | awk '{print $$2}'`

_WGET:=		${shell type wget | awk '{if ($$0 ~ /wget is/) print $$3}'}

_DWN_LOCALPKG=	${abspath ${DWN_LOCALPKG}}

mksum:
	@echo ${_HASH}

ifeq (${_WGET},)
ERRORS+=	"wget program not found."
endif

ifndef DWN_PKGNAME
ERRORS+=	"DWN_PKGNAME not defined."
endif

ifndef DWN_PKGURL
ERRORS+=	"DWN_PKGURL not defined."
endif

ifndef DWN_PKGTAR
ERRORS+=	"DWN_PKGTAR not defined."
endif

# FIXME 
ifndef DWN_HASH
ERROR+=		"DWN_HASH not defined."
else
_CORRECT_HASH:=	${shell [ -f ${DWN_HASH} ] && cat ${DWN_HASH}}
endif

ifdef ERRORS
${DWN_UNPACKEDIR}:
	@echo ${ERRORS}
	@exit 1
else

.PHONY: download extract mksum

extract: ${DWN_UNPACKEDIR}

${DWN_UNPACKEDIR}: ${_DWN_LOCALPKG}
	@[ "${_HASH}" = "${_CORRECT_HASH}" ] || \
		(echo "${DWN_LOCALPKG}: Invalid Signature"; exit 1)
	[ -d ${DWN_UNTARDIR} ] || ${MKDIR} -p ${DWN_UNTARDIR}
	cd ${DWN_UNTARDIR} && ${DWN_UNTAR} ${_DWN_LOCALPKG}

download: ${_DWN_LOCALPKG}

${_DWN_LOCALPKG}:
	${_WGET} -P ${DWN_PKGDIR} ${DWN_PKGURL}

endif
