#
# Copyright 2007-2010 Claudio Castiglia
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
# BIN=		List of binaries
# DOC=		List of documents
# INC=		List of includes
# LIB=		List of libraries (.a)
# MKF=		List of files to include into the mk/ structure
# MOD=		List of modules (.pld)
# SHLIB=	List of modules (.so)
# PREFIX=	Base directory
# BINDIR=	Binaries directory
# DOCDIR=	Document directory
# INCDIR=	Includes directory
# LIBDIR=	Libraries directory
#

#
# Strip directories from file names.
#
_STBIN=	${notdir ${BIN}}
_STDOC=	${notdir ${DOC}}
_STINC=	${notdir ${INC}}
_STLIB=	${notdir ${LIB}}
_STMKF=	${notdir ${MKF}}
_STMOD=	${notdir ${MOD}}
_STSHL=	${notdir ${SHLIB}}

MODDIR=	${LIBDIR}

#
# Create destination fullpaths.
#
DSTBIN=	${_STBIN:%=${BINDIR}/%}
DSTDOC=	${_STDOC:%=${DOCDIR}/%}
DSTINC=	${_STINC:%=${INCDIR}/%}
DSTLIB=	${_STLIB:%=${LIBDIR}/%}
DSTMKF=	${_STMKF:%=${MKFDIR}/%}
DSTMOD=	${_STMOD:%=${MODDIR}/%}
DSTSHL=	${_STSHL:%=${MODDIR}/%}

_FILES=	${DSTBIN} ${DSTDOC} ${DSTINC} ${DSTLIB} ${DSTMKF} ${DSTMOD} ${DSTSHL}
_DIRS=	${BINDIR} ${DOCDIR} ${INCDIR} ${LIBDIR} ${MKFDIR}

ifeq (${_FILES}, "")
install:
else
install: ${_FILES}

ifdef BIN
${DSTBIN}: ${BIN} ${BINDIR}
	${INSTALL} -o ${OWNER_BIN} -g ${GROUP_BIN} -m ${MODE_BIN} \
		${BIN} ${BINDIR}
endif

ifdef DOC
${DSTDOC}: ${DOC} ${DOCDIR}
	${INSTALL} -o ${OWNER_DATA} -g ${GROUP_DATA} -m ${MODE_DATA} \
		${DOC} ${DOCDIR}
endif

ifdef INC
${DSTINC}: ${INC} ${INCDIR}
	${INSTALL} -o ${OWNER_DATA} -g ${GROUP_DATA} -m ${MODE_DATA} \
		${INC} ${INCDIR}
endif

ifdef LIB
${DSTLIB}: ${LIB} ${LIBDIR}
	${INSTALL} -o ${OWNER_DATA} -g ${GROUP_DATA} -m ${MODE_DATA} \
		${LIB} ${LIBDIR}
endif

ifdef MKF
${DSTMKF}: ${MKF} ${MKFDIR}
	${INSTALL} -o ${OWNER_DATA} -g ${GROUP_DATA} -m ${MODE_DATA} \
		${MKF} ${MKFDIR}
endif

ifdef MOD
${DSTMOD}: ${MOD} ${MODDIR}
	${INSTALL} -o ${OWNER_DATA} -g ${GROUP_DATA} -m ${MODE_DATA} \
		${MOD} ${MODDIR}
endif

ifdef SHLIB
${DSTSHL}: ${SHLIB} ${MODDIR}
	${INSTALL} -o ${OWNER_DATA} -g ${GROUP_DATA} -m ${MODE_DATA} \
		${SHLIB} ${MODDIR}
endif
endif

${_DIRS}:
	${INSTALL} -d -o ${OWNER_DIR} -g ${GROUP_DIR} -m ${MODE_DIR} $@

