#
# Copyright 2007-2010 Claudio Castiglia
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
# libtarget.mk is used to generate either:
# 	* A peludo module (.pld)
# 	* A peluded shared library (.so)
# 	* A peluded object library (.a)
#
# This file IS NOT used to generate the MAIN peludo module (a module
# containing the main() function), use bintarget.mk for that.
#
# Maybe you want to include objtarget.mk too.
#
# MOD=			Name of the Peludo Module to generate.
#			Define it when you want a .pld file.
#
# MODNAME=		Name of the Peludo Module once it is loaded.
# 			Default is ${notdir ${MOD}}.
#
# SHLIB=		Name of Shared Library to generate.
# 			Define it when you want an ELF .so file
# 			(If you define MOD but not SHLIB then a default
# 			SHLIB will be generated as ${MOD}.so).
#
# LIB=		 	Name of the .a library to generate.
# 			Define it when you want a .a file.
#
# OBJS_ALL=	 	List of object files.
# 			(defined automatically by objtarget.mk).
#
# FIND_OBJS=	 	List of directories where to search for object files.
# 			Define this when you don't know in advance the
# 			object files to include into the generated library.
# 
# LDFLAGS=		Linker flags.
# 			(Defined automatically by base.mk).
#
# ARADD=	 	List of additional .a libraries to include into
# 			the generated file.
#
# LDADD=		Additional shared libraries to link against when
# 			generating a .so.
#
# NOMOD=		If this variable is defined no .pld nor .so files
# 			are generated no matter whether MOD or SHLIB are
# 			defined or not.
#
#
ifdef FIND_OBJS
_FIND_OBJS=	${shell find ${FIND_OBJS} -name "*.o"}
endif

ifdef LIB
CLEANFILES+=	${LIB}
_LIBTARGET+=	${LIB}

${LIB}: ${OBJS_ALL}
	${AR} ${ARFLAGS} $@ ${OBJS_ALL} ${ARADD} ${_FIND_OBJS}
	${RANLIB} $@
endif

#
# SHLIB and MOD should be generated when NOMOD is not defined.
#
ifndef NOMOD

ifdef SHLIB
_SHLIB=		${SHLIB}
endif

ifdef MOD
CLEANFILES+=	${MOD}
_LIBTARGET+=	${MOD}
ifndef SHLIB
_SHLIB=		${MOD}.so
endif
endif

ifdef _SHLIB
CLEANFILES+=	${_SHLIB}
_LIBTARGET+=	${_SHLIB}

${_SHLIB}: ${OBJS_ALL}
	${LD} ${LDFLAGS} -o $@ ${LDADD} $+ ${ARADD} ${_FIND_OBJS}
endif

ifdef MOD
MODNAME?=	${notdir ${MOD}}

${MOD}: ${_SHLIB}
	${ELF2PLD} ${ELF2PLD_FLAGS} -p ${MODNAME} $< $@
endif

endif

libtarget: ${_LIBTARGET}

