#
# Copyright 2007-2010 Claudio Castiglia
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
# plbtarget.mk is used to generate PLB files.
#
# PLB=		Name of the .plb file to generate.
#		MANDATORY variable.
#
# PLBELF=	If this variable is set an ELF is also generated
#		embedding the PLB.
#
# MODS=		List of modules to include into the PLB file
#		(in dependency order).
# 		MANDATORY variable.
#
# PLBOOT=	Bootstrap code.
#		(Default is ${LIBDIR}/krn0_${PLTARGET}.plo)
#
# PLKERNEL=	Kernel code.
#		(Default is ${LIBDIR}/krn1_${PLTARGET}.plo)
#
# LIBC=		C runtime.
#		(Default is ${LIBDIR}/libplc_${PLTARGET}.pld)
#
# LIBPTHREAD=	Multithread library.
#		(Default is ${LIBDIR}/libpthread_${PLTARGET}.pld)
#
ifndef PLB
PLERROR+=	"PLB not defined."
endif

ifndef MODS
PLERROR+=	"MODS not defined."
endif

PLBOOT?=	${LIBDIR}/krn0_${PLTARGET}.plo
PLKERNEL?=	${LIBDIR}/krn1_${PLTARGET}.plo
PLENDM?=	${LIBDIR}/end_${PLTARGET}.pld

LIBC?=		${LIBDIR}/libplc_${PLTARGET}.pld
LIBPTHREAD?=	${LIBDIR}/libpthread_${PLTARGET}.pld

.PHONY: plb plbelf

ifndef PLERROR
CLEANFILES+=	${PLB}

plb: ${PLB}

${PLB}: ${PLBOOT} ${PLKERNEL} ${MODS} ${PLENDM}
	cat $^ > $@

ifdef PLBELF
CLEANFILES+=	${PLBELF}
${PLBELF}: ${PLB}
	${PLB2ELF} -p ${PLTARGET} $< $@

endif

else
plb plbelf:

endif

