/*
 * Copyright 2007-2010 Claudio Castiglia
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/**
 * @file src/core/kernel/test/ktest.c
 * Little program to test the kernel code.
 */
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/mman.h>
#include <err.h>
#include <fcntl.h>
#include <libgen.h>
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>

#include <archdef.h>
#include <pldef.h>
#include <plist.h>
#include <pld.h>

#include <mm.h>
#include <kernel.h>

#include "debug.h"


static char *progname;


int main(int argc, char **argv)
{
	PLSTATUS status;
	struct stat st;
	void *map;
	int fd;

	progname = basename(argv[0]);

	if (argc < 2) {
		errx(-1, "usage: %s <pld> [<calling_mode>]", progname);
		/* NOTREACHED */
	}
	plcalling_mode = argc == 2 ? 0 : atoi(argv[2]);

	fd = open(argv[1], O_RDONLY);
	if (fd < 0) {
		err(-1, "%s: Can't open", argv[1]);
		/* NOTREACHED */
	}

	if (fstat(fd, &st) < 0) {
		err(-1, " %s: Can't stat", argv[1]);
		/* NOTREACHED */
	}

	map = mmap(NULL, st.st_size, PROT_READ | PROT_EXEC, MAP_PRIVATE, fd, 0);
	if (map == MAP_FAILED) {
		err(-1, "%s: Can't mmap", argv[1]);
		/* NOTREACHED */
	}

#if 1
	mm_init();

	pld_init(NULL);
	//status = pld_open(NULL, map);
	status = pld_open_all(map);
	pld_release();

	mm_release();
#else
	kinit(map, NULL, NULL, NULL);
	krelease()
#endif

	(void)munmap(map, st.st_size);
	(void)close(fd);

	printf("\n%s: pld_open exited with status: %d (%s)\n", progname,
	    status, plstrerror(status));

	return (status);
}


