/*
 * Copyright 2007-2010 Claudio Castiglia
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/**
 * @file include/std/dirent.h
 * Peludo LibC - dirent.h
 */
#ifndef __DIRENT_H__
#define __DIRENT_H__

#include <plc.h>


typedef long DIR;


static inline DIR __attribute__((always_inline)) *
opendir(const char *path)
{
	long handle;

	switch (plsys_opendir(&handle, path)) {
	case PLSTATUS_SUCCESS:
		return ((DIR *)handle);

	case PLSTATUS_INVALID:
		errno = EINVAL;
		break;

	default:;
	}

	return NULL;
}


static inline int __attribute__((always_inline))
closedir(DIR *dir)
{
	plsys_closedir((long)dir);
	return 0;
}


static inline struct dirent __attribute__((always_inline)) *
readdir(DIR *dir)
{
	struct dirent *de;

	return ((plsys_readdir((long)dir, &de) == PLSTATUS_SUCCESS) ?
	    de : NULL);
}


#endif	/* __DIRENT_H__ */
