/*
 * Copyright 2007-2010 Claudio Castiglia
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/**
 * @file include/std/semaphore.h
 * Peludo Threads - Semaphores.
 * @see http://www.opengroup.org/onlinepubs/007908775/xsh/semaphore.h.html
 */
#ifndef __SEMAPHORE_H__
#define __SEMAPHORE_H__

#include <plspinlock.h>

#define _POSIX_SEMAPHORES

struct _sem {
	PLOCK	sem_lock;
	long	sem_wcount;
	long	sem_value;
};

typedef struct _sem * sem_t;

#define SEM_FAILED	((sem_t *)0)
#define SEM_VALUE_MAX	(2147483647)

int	 sem_init(sem_t *, int, unsigned int);
int	 sem_destroy(sem_t *);
int	 sem_getvalue(sem_t *, int *);
int	 sem_post(sem_t *);
int	 sem_trywait(sem_t *);
int	 sem_wait(sem_t *);

/*
 * Not implemented.
 */
sem_t	*sem_open(const char *, int, ...);
int	 sem_close(sem_t *);
int	 sem_unlink(const char *);

#endif	/* __SEMAPHORE_H__ */
