/*
 * Copyright 2007-2010 Claudio Castiglia
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
#ifndef _SIGNAL_H_
#define _SIGNAL_H_

#include <plc.h>
#include <pthread.h>


int kill(pid_t, int);
int raise(int);

int pthread_kill(pthread_t, int);
int pthread_killall_np(int);
int pthread_sigmask(int, const sigset_t *, sigset_t *);

int sigsuspend(const sigset_t *);
int sigwait(const sigset_t *, int *);


static inline int __attribute__((always_inline))
sigaddset(sigset_t *set, int signo)
{
	return ((plsys_sigaddset(set, signo) == PLSTATUS_SUCCESS) ? 0 : -1);
}


static inline int __attribute__((always_inline))
sigdelset(sigset_t *set, int signo)
{
	return ((plsys_sigdelset(set, signo) == PLSTATUS_SUCCESS) ? 0 : -1);
}


static inline int __attribute__((always_inline))
sigemptyset(sigset_t *set)
{
	return ((plsys_sigemptyset(set) == PLSTATUS_SUCCESS) ? 0 : -1);
}


static inline int __attribute__((always_inline))
sigfillset(sigset_t *set)
{
	return ((plsys_sigfillset(set) == PLSTATUS_SUCCESS) ? 0 : -1);
}


static inline int __attribute__((always_inline))
sigismember(const sigset_t *set, int signo)
{
	return plsys_sigismember(set, signo);
}


static inline int __attribute__((always_inline))
sigaction(int signo, const struct sigaction *act, struct sigaction *oact)
{
	return ((plsys_sigaction(signo, act, oact) == PLSTATUS_SUCCESS) ?
	    0 : -1);
}


static inline int __attribute__((always_inline))
sigprocmask(int how, const sigset_t *newmask, sigset_t *oldmask)
{
        return ((plsys_tsigmask(how, newmask, oldmask) == PLSTATUS_SUCCESS)
	    ? 0 : errno);
}


#endif	/* _SIGNALH_ */
