/*
 * Copyright 2007-2010 Claudio Castiglia
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/**
 * @file include/std/stdio.h
 * Peludo LibC - stdio.h
 */
#ifndef __STDIO_H__
#define __STDIO_H__

#include <plarchdef.h>
#include <stdarg.h>
#include <limits.h>

#define BUFSIZ		1024

#define EOF		(-1)

#define FILENAME_MAX	PATH_MAX

#define MAXPATHLEN	PATH_MAX	/* XXX sys/param.h? */

typedef	struct _FILE {
	long	f_fd;
} FILE;

extern FILE *stdin;
extern FILE *stdout;
extern FILE *stderr;

FILE	*fopen(const char *, const char *);
FILE	*fdopen(int, const char *);
int	 ferror(FILE *);
int	 fclose(FILE *);
int	 fflush(FILE *);
int	 fileno(FILE *);
size_t	 fread(void *, size_t, size_t, FILE *);
size_t	 fwrite(const void *, size_t, size_t, FILE *);

void	 perror(const char *);

/* XXX
 * plprint() family does not support all printf() format options.
 * See src/lib/plc/plvnprint_cb.c
 */

int	 fprintf(FILE *, const char *, ...);
int	 vfprintf(FILE *, const char *, va_list);

#define	printf(args...)		fprintf(stdout, args)
#define vprintf(args...)	vfprintf(stdout, args)

#define sprintf			plsprint
#define vsprintf		plvsprint

#define snprintf		plsnprint
#define vsnprintf		plvsnprint

int	 remove(const char *);
int	 rename(const char *, const char *);


#endif	/* __STDIO_H__ */
