/*
 * Copyright 2007-2010 Claudio Castiglia
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
#ifndef _STDLIB_H_
#define _STDLIB_H_

#include <plc.h>
#include <pllog.h>
#include <sys/types.h>

#define RAND_MAX		(0x7fffffff)

#define alloca(size)		__builtin_alloca((size))
#define realloc(ptr, size)	plrealloc((ptr), (size))

quad_t	 __divdi3(quad_t, quad_t);
quad_t	 __moddi3(quad_t, quad_t);
u_quad_t __qdivrem(u_quad_t, u_quad_t, u_quad_t *);

int atoi(const char *);
void	*calloc(size_t, size_t);
void	 free(void *);
void	*malloc(size_t);
void	 qsort(void *, size_t, size_t, int (*)(const void *, const void *));
int	 rand();
int	 rand_r();
char	*realpath(const char *, char *);
void	 srand(unsigned int);
long	 strtol(const char *, char **, int);


static inline void __attribute__((always_inline))
abort()
{
	pldebug("Aborted\n");
	plsys_exit(-1);
}


static inline void __attribute__((always_inline))
exit(int status)
{
	plexit((long)status);
}


static inline char __attribute__((always_inline)) *
getenv(const char *name)
{
	return NULL;
}


#endif	/* _STDLIB_H_ */
