/*
 * Copyright 2007-2010 Claudio Castiglia
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/**
 * Peludo libc.
 */
#ifndef _SYS_SOCKET_H_
#define _SYS_SOCKET_H_

#include <plsock.h>

/*
 * XXX with the fd manager the SOCKET define is not needed
 * and all unix/posix functions can be easily ported to windows.
 */
#ifndef PLOS_win32
#define SOCKET		int
#define SOCKET_ERROR	-1
#define INVALID_SOCKET	-1
#endif


SOCKET	accept(SOCKET, struct sockaddr *, socklen_t *);
SOCKET	connect(SOCKET, const struct sockaddr *, socklen_t);
ssize_t	recv(SOCKET, void *, size_t, int);
ssize_t	recvfrom(SOCKET, void *, size_t, int, struct sockaddr *, socklen_t *);
ssize_t	recvmsg(SOCKET, struct msghdr *, int);
ssize_t	send(SOCKET, const void *, size_t, int);
ssize_t	sendto(SOCKET, const void *, size_t, int, const struct sockaddr *,
	socklen_t);
ssize_t	sendmsg(SOCKET, const struct msghdr *, int);
int	socketpair(int, int, int, SOCKET[2]);


static inline int __attribute__((always_inline))
bind(SOCKET fd, const struct sockaddr *addr, socklen_t len)
{
	return (plsock_bind((long)fd, addr, len) == PLSTATUS_SUCCESS
	    ? 0 : SOCKET_ERROR);
}


static inline int __attribute__((always_inline))
getpeername(SOCKET sock, struct sockaddr *addr, socklen_t *len)
{
	return ((plsock_getpeername((long)sock, addr, len)
	    == PLSTATUS_SUCCESS) ? 0 : SOCKET_ERROR);
}


static inline int __attribute__((always_inline))
getsockname(SOCKET sock, struct sockaddr *addr, socklen_t *len)
{
	return ((plsock_getsockname((long)sock, addr, len)
	    == PLSTATUS_SUCCESS) ? 0 : SOCKET_ERROR);
}


static inline int __attribute__((always_inline))
getsockopt(SOCKET sock, int level, int optname, void *optval, socklen_t *optlen)
{
	return ((plsock_getsockopt((long)sock, level, optname, optval, optlen)
	    == PLSTATUS_SUCCESS) ? 0 : SOCKET_ERROR);
}


static inline int __attribute__((always_inline))
listen(SOCKET fd, int backlog)
{
	return (plsock_listen((long)fd, backlog) == PLSTATUS_SUCCESS
	    ? 0 : SOCKET_ERROR);
}


static inline int __attribute__((always_inline))
setsockopt(SOCKET sock, int level, int optname, const void *optval,
    socklen_t optlen)
{
	return ((plsock_setsockopt((long)sock, level, optname, optval, optlen)
	    == PLSTATUS_SUCCESS) ? 0 : SOCKET_ERROR);
}


static inline int __attribute__((always_inline))
shutdown(SOCKET sock, int how)
{
	return ((plsock_shutdown((long)sock, how)) == PLSTATUS_SUCCESS
	    ? 0 : SOCKET_ERROR);
}


static inline SOCKET __attribute__((always_inline))
socket(int af, int type, int proto)
{
	long sock;

	return ((plsock_socket(&sock, af, type, proto) == PLSTATUS_SUCCESS)
	    ? (SOCKET)sock : SOCKET_ERROR);
}


#endif	/* _SYS_SOCKET_H_ */
