/*
 * Copyright 2007-2010 Claudio Castiglia
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/**
 * @file src/include/sysdep/freebsd_i386.h
 * FreeBSD/i386 specific.
 */
#ifndef FREEBSD_I386_H
#define FREEBSD_I386_H


/***********************************************************
 * VARIOUS (needed here, others use them)
 ***********************************************************/
typedef uint32		dev_t;
typedef uint32		gid_t;
typedef uint32		ino_t;
typedef uint16		mode_t;
typedef uint16		nlink_t;
typedef uint64		off_t;
typedef uint32		pid_t;
typedef unsigned long	time_t;
typedef uint32		uid_t;


/***********************************************************
 * LIMITS
 ***********************************************************/
#define NAME_MAX			255
#define MAXSYMLINKS			 20
#define HOST_NAME_MAX			 64
#define MAXHOSTNAMELEN			 HOST_NAME_MAX


/***********************************************************
 * SYSCALLS
 ***********************************************************/
#define	SYS_syscall			  0
#define	SYS_exit			  1
#define	SYS_fork			  2
#define	SYS_read			  3
#define	SYS_write			  4
#define	SYS_open			  5
#define	SYS_close			  6
#define	SYS_wait4			  7
				/* 8 is old creat */
#define	SYS_link			  9
#define	SYS_unlink			 10
				/* 11 is obsolete execv */
#define	SYS_chdir			 12
#define	SYS_fchdir			 13
#define	SYS_mknod			 14
#define	SYS_chmod			 15
#define	SYS_chown			 16
#define	SYS_break			 17
#define	SYS_freebsd4_getfsstat		 18
				/* 19 is old lseek */
#define	SYS_getpid			 20
#define	SYS_mount			 21
#define	SYS_unmount			 22
#define	SYS_setuid			 23
#define	SYS_getuid			 24
#define	SYS_geteuid			 25
#define	SYS_ptrace			 26
#define	SYS_recvmsg			 27
#define	SYS_sendmsg			 28
#define	SYS_recvfrom			 29
#define	SYS_accept			 30
#define	SYS_getpeername			 31
#define	SYS_getsockname			 32
#define	SYS_access			 33
#define	SYS_chflags			 34
#define	SYS_fchflags			 35
#define	SYS_sync			 36
#define	SYS_kill			 37
				/* 38 is old stat */
#define	SYS_getppid			 39
				/* 40 is old lstat */
#define	SYS_dup				 41
#define	SYS_pipe			 42
#define	SYS_getegid			 43
#define	SYS_profil			 44
#define	SYS_ktrace			 45
				/* 46 is old sigaction */
#define	SYS_getgid			 47
				/* 48 is old sigprocmask */
#define	SYS_getlogin			 49
#define	SYS_setlogin			 50
#define	SYS_acct			 51
				/* 52 is old sigpending */
#define	SYS_sigaltstack			 53
#define	SYS_ioctl			 54
#define	SYS_reboot			 55
#define	SYS_revoke			 56
#define	SYS_symlink			 57
#define	SYS_readlink			 58
#define	SYS_execve			 59
#define	SYS_umask			 60
#define	SYS_chroot			 61
				/* 62 is old fstat */
				/* 63 is old getkerninfo */
				/* 64 is old getpagesize */
#define	SYS_msync			 65
#define	SYS_vfork			 66
				/* 67 is obsolete vread */
				/* 68 is obsolete vwrite */
#define	SYS_sbrk			 69
#define	SYS_sstk			 70
				/* 71 is old mmap */
#define	SYS_vadvise			 72
#define	SYS_munmap			 73
#define	SYS_mprotect			 74
#define	SYS_madvise			 75
				/* 76 is obsolete vhangup */
				/* 77 is obsolete vlimit */
#define	SYS_mincore			 78
#define	SYS_getgroups			 79
#define	SYS_setgroups			 80
#define	SYS_getpgrp			 81
#define	SYS_setpgid			 82
#define	SYS_setitimer			 83
				/* 84 is old wait */
#define	SYS_swapon			 85
#define	SYS_getitimer			 86
				/* 87 is old gethostname */
				/* 88 is old sethostname */
#define	SYS_getdtablesize		 89
#define	SYS_dup2			 90
#define	SYS_fcntl			 92
#define	SYS_select			 93
#define	SYS_fsync			 95
#define	SYS_setpriority			 96
#define	SYS_socket			 97
#define	SYS_connect			 98
				/* 99 is old accept */
#define	SYS_getpriority			100
				/* 101 is old send */
				/* 102 is old recv */
				/* 103 is old sigreturn */
#define	SYS_bind			104
#define	SYS_setsockopt			105
#define	SYS_listen			106
				/* 107 is obsolete vtimes */
				/* 108 is old sigvec */
				/* 109 is old sigblock */
				/* 110 is old sigsetmask */
				/* 111 is old sigsuspend */
				/* 112 is old sigstack */
				/* 113 is old recvmsg */
				/* 114 is old sendmsg */
				/* 115 is obsolete vtrace */
#define	SYS_gettimeofday		116
#define	SYS_getrusage			117
#define	SYS_getsockopt			118
#define	SYS_readv			120
#define	SYS_writev			121
#define	SYS_settimeofday		122
#define	SYS_fchown			123
#define	SYS_fchmod			124
				/* 125 is old recvfrom */
#define	SYS_setreuid			126
#define	SYS_setregid			127
#define	SYS_rename			128
				/* 129 is old truncate */
				/* 130 is old ftruncate */
#define	SYS_flock			131
#define	SYS_mkfifo			132
#define	SYS_sendto			133
#define	SYS_shutdown			134
#define	SYS_socketpair			135
#define	SYS_mkdir			136
#define	SYS_rmdir			137
#define	SYS_utimes			138
				/* 139 is obsolete 4.2 sigreturn */
#define	SYS_adjtime			140
				/* 141 is old getpeername */
				/* 142 is old gethostid */
				/* 143 is old sethostid */
				/* 144 is old getrlimit */
				/* 145 is old setrlimit */
				/* 146 is old killpg */
#define	SYS_setsid			147
#define	SYS_quotactl			148
				/* 149 is old quota */
				/* 150 is old getsockname */
#define	SYS_nlm_syscall			154
#define	SYS_nfssvc			155
				/* 156 is old getdirentries */
#define	SYS_freebsd4_statfs		157
#define	SYS_freebsd4_fstatfs		158
#define	SYS_lgetfh			160
#define	SYS_getfh			161
#define	SYS_getdomainname		162
#define	SYS_setdomainname		163
#define	SYS_uname			164
#define	SYS_sysarch			165
#define	SYS_rtprio			166
#define	SYS_semsys			169
#define	SYS_msgsys			170
#define	SYS_shmsys			171
#define	SYS_freebsd6_pread		173
#define	SYS_freebsd6_pwrite		174
#define	SYS_setfib			175
#define	SYS_ntp_adjtime			176
#define	SYS_setgid			181
#define	SYS_setegid			182
#define	SYS_seteuid			183
#define	SYS_stat			188
#define	SYS_fstat			189
#define	SYS_lstat			190
#define	SYS_pathconf			191
#define	SYS_fpathconf			192
#define	SYS_getrlimit			194
#define	SYS_setrlimit			195
#define	SYS_getdirentries		196
#define	SYS_freebsd6_mmap		197
#define	SYS___syscall			198
#define	SYS_freebsd6_lseek		199
#define	SYS_freebsd6_truncate		200
#define	SYS_freebsd6_ftruncate		201
#define	SYS___sysctl			202
#define	SYS_mlock			203
#define	SYS_munlock			204
#define	SYS_undelete			205
#define	SYS_futimes			206
#define	SYS_getpgid			207
#define	SYS_poll			209
#define	SYS___semctl			220
#define	SYS_semget			221
#define	SYS_semop			222
#define	SYS_msgctl			224
#define	SYS_msgget			225
#define	SYS_msgsnd			226
#define	SYS_msgrcv			227
#define	SYS_shmat			228
#define	SYS_shmctl			229
#define	SYS_shmdt			230
#define	SYS_shmget			231
#define	SYS_clock_gettime		232
#define	SYS_clock_settime		233
#define	SYS_clock_getres		234
#define	SYS_ktimer_create		235
#define	SYS_ktimer_delete		236
#define	SYS_ktimer_settime		237
#define	SYS_ktimer_gettime		238
#define	SYS_ktimer_getoverrun		239
#define	SYS_nanosleep			240
#define	SYS_ntp_gettime			248
#define	SYS_minherit			250
#define	SYS_rfork			251
#define	SYS_openbsd_poll		252
#define	SYS_issetugid			253
#define	SYS_lchown			254
#define	SYS_aio_read			255
#define	SYS_aio_write			256
#define	SYS_lio_listio			257
#define	SYS_getdents			272
#define	SYS_lchmod			274
#define	SYS_netbsd_lchown		275
#define	SYS_lutimes			276
#define	SYS_netbsd_msync		277
#define	SYS_nstat			278
#define	SYS_nfstat			279
#define	SYS_nlstat			280
#define	SYS_preadv			289
#define	SYS_pwritev			290
#define	SYS_freebsd4_fhstatfs		297
#define	SYS_fhopen			298
#define	SYS_fhstat			299
#define	SYS_modnext			300
#define	SYS_modstat			301
#define	SYS_modfnext			302
#define	SYS_modfind			303
#define	SYS_kldload			304
#define	SYS_kldunload			305
#define	SYS_kldfind			306
#define	SYS_kldnext			307
#define	SYS_kldstat			308
#define	SYS_kldfirstmod			309
#define	SYS_getsid			310
#define	SYS_setresuid			311
#define	SYS_setresgid			312
				/* 313 is obsolete signanosleep */
#define	SYS_aio_return			314
#define	SYS_aio_suspend			315
#define	SYS_aio_cancel			316
#define	SYS_aio_error			317
#define	SYS_oaio_read			318
#define	SYS_oaio_write			319
#define	SYS_olio_listio			320
#define	SYS_yield			321
				/* 322 is obsolete thr_sleep */
				/* 323 is obsolete thr_wakeup */
#define	SYS_mlockall			324
#define	SYS_munlockall			325
#define	SYS___getcwd			326
#define	SYS_sched_setparam		327
#define	SYS_sched_getparam		328
#define	SYS_sched_setscheduler		329
#define	SYS_sched_getscheduler		330
#define	SYS_sched_yield			331
#define	SYS_sched_get_priority_max	332
#define	SYS_sched_get_priority_min	333
#define	SYS_sched_rr_get_interval	334
#define	SYS_utrace			335
#define	SYS_freebsd4_sendfile		336
#define	SYS_kldsym			337
#define	SYS_jail			338
#define	SYS_sigprocmask			340
#define	SYS_sigsuspend			341
#define	SYS_freebsd4_sigaction		342
#define	SYS_sigpending			343
#define	SYS_freebsd4_sigreturn		344
#define	SYS_sigtimedwait		345
#define	SYS_sigwaitinfo			346
#define	SYS___acl_get_file		347
#define	SYS___acl_set_file		348
#define	SYS___acl_get_fd		349
#define	SYS___acl_set_fd		350
#define	SYS___acl_delete_file		351
#define	SYS___acl_delete_fd		352
#define	SYS___acl_aclcheck_file		353
#define	SYS___acl_aclcheck_fd		354
#define	SYS_extattrctl			355
#define	SYS_extattr_set_file		356
#define	SYS_extattr_get_file		357
#define	SYS_extattr_delete_file		358
#define	SYS_aio_waitcomplete		359
#define	SYS_getresuid			360
#define	SYS_getresgid			361
#define	SYS_kqueue			362
#define	SYS_kevent			363
#define	SYS_extattr_set_fd		371
#define	SYS_extattr_get_fd		372
#define	SYS_extattr_delete_fd		373
#define	SYS___setugid			374
#define	SYS_nfsclnt			375
#define	SYS_eaccess			376
#define	SYS_nmount			378
#define	SYS_kse_exit			379
#define	SYS_kse_wakeup			380
#define	SYS_kse_create			381
#define	SYS_kse_thr_interrupt		382
#define	SYS_kse_release			383
#define	SYS___mac_get_proc		384
#define	SYS___mac_set_proc		385
#define	SYS___mac_get_fd		386
#define	SYS___mac_get_file		387
#define	SYS___mac_set_fd		388
#define	SYS___mac_set_file		389
#define	SYS_kenv			390
#define	SYS_lchflags			391
#define	SYS_uuidgen			392
#define	SYS_sendfile			393
#define	SYS_mac_syscall			394
#define	SYS_getfsstat			395
#define	SYS_statfs			396
#define	SYS_fstatfs			397
#define	SYS_fhstatfs			398
#define	SYS_ksem_close			400
#define	SYS_ksem_post			401
#define	SYS_ksem_wait			402
#define	SYS_ksem_trywait		403
#define	SYS_ksem_init			404
#define	SYS_ksem_open			405
#define	SYS_ksem_unlink			406
#define	SYS_ksem_getvalue		407
#define	SYS_ksem_destroy		408
#define	SYS___mac_get_pid		409
#define	SYS___mac_get_link		410
#define	SYS___mac_set_link		411
#define	SYS_extattr_set_link		412
#define	SYS_extattr_get_link		413
#define	SYS_extattr_delete_link		414
#define	SYS___mac_execve		415
#define	SYS_sigaction			416
#define	SYS_sigreturn			417
#define	SYS_getcontext			421
#define	SYS_setcontext			422
#define	SYS_swapcontext			423
#define	SYS_swapoff			424
#define	SYS___acl_get_link		425
#define	SYS___acl_set_link		426
#define	SYS___acl_delete_link		427
#define	SYS___acl_aclcheck_link		428
#define	SYS_sigwait			429
#define	SYS_thr_create			430
#define	SYS_thr_exit			431
#define	SYS_thr_self			432
#define	SYS_thr_kill			433
#define	SYS__umtx_lock			434
#define	SYS__umtx_unlock		435
#define	SYS_jail_attach			436
#define	SYS_extattr_list_fd		437
#define	SYS_extattr_list_file		438
#define	SYS_extattr_list_link		439
#define	SYS_kse_switchin		440
#define	SYS_ksem_timedwait		441
#define	SYS_thr_suspend			442
#define	SYS_thr_wake			443
#define	SYS_kldunloadf			444
#define	SYS_audit			445
#define	SYS_auditon			446
#define	SYS_getauid			447
#define	SYS_setauid			448
#define	SYS_getaudit			449
#define	SYS_setaudit			450
#define	SYS_getaudit_addr		451
#define	SYS_setaudit_addr		452
#define	SYS_auditctl			453
#define	SYS__umtx_op			454
#define	SYS_thr_new			455
#define	SYS_sigqueue			456
#define	SYS_kmq_open			457
#define	SYS_kmq_setattr			458
#define	SYS_kmq_timedreceive		459
#define	SYS_kmq_timedsend		460
#define	SYS_kmq_notify			461
#define	SYS_kmq_unlink			462
#define	SYS_abort2			463
#define	SYS_thr_set_name		464
#define	SYS_aio_fsync			465
#define	SYS_rtprio_thread		466
#define	SYS_sctp_peeloff		471
#define	SYS_sctp_generic_sendmsg	472
#define	SYS_sctp_generic_sendmsg_iov	473
#define	SYS_sctp_generic_recvmsg	474
#define	SYS_pread			475
#define	SYS_pwrite			476
#define	SYS_mmap			477
#define	SYS_lseek			478
#define	SYS_truncate			479
#define	SYS_ftruncate			480
#define	SYS_thr_kill2			481
#define	SYS_cpuset			484
#define	SYS_cpuset_setid		485
#define	SYS_cpuset_getid		486
#define	SYS_cpuset_getaffinity		487
#define	SYS_cpuset_setaffinity		488
#define	SYS_MAXSYSCALL			489


/***********************************************************
 * ACCESS
 ***********************************************************/
#define F_OK				0
#define X_OK				1
#define W_OK				2
#define R_OK				4


/***********************************************************
 * DIRENT
 ***********************************************************/
/* posix dirent */
/*
struct dirent {
	ino_t		d_ino;
	char		d_name[NAME_MAX + 1];
};
*/

/* freebsd dirent */
struct freebsd_dirent {
	unsigned int	d_fileno;
	unsigned short	d_reclen;
	unsigned char	d_type;
	unsigned char	d_namlen;
	char		d_name[NAME_MAX + 1];
};

#define	dirent				freebsd_dirent


/***********************************************************
 * ERRNO
 ***********************************************************/
#define	EPERM				 1
#define	ENOENT				 2
#define	ESRCH				 3
#define	EINTR				 4
#define	EIO				 5
#define	ENXIO				 6
#define	E2BIG				 7
#define	ENOEXEC				 8
#define	EBADF				 9
#define	ECHILD				10
#define	EDEADLK				11
					/* 11 was EAGAIN */
#define	ENOMEM				12
#define	EACCES				13
#define	EFAULT				14
#define	ENOTBLK				15
#define	EBUSY				16
#define	EEXIST				17
#define	EXDEV				18
#define	ENODEV				19
#define	ENOTDIR				20
#define	EISDIR				21
#define	EINVAL				22
#define	ENFILE				23
#define	EMFILE				24
#define	ENOTTY				25
#define	ETXTBSY				26
#define	EFBIG				27
#define	ENOSPC				28
#define	ESPIPE				29
#define	EROFS				30
#define	EMLINK				31
#define	EPIPE				32
#define	EDOM				33
#define	ERANGE				34
#define	EAGAIN				35
#define	EWOULDBLOCK			EAGAIN
#define	EINPROGRESS			36
#define	EALREADY			37
#define	ENOTSOCK			38
#define	EDESTADDRREQ			39
#define	EMSGSIZE			40
#define	EPROTOTYPE			41
#define	ENOPROTOOPT			42
#define	EPROTONOSUPPORT			43
#define	ESOCKTNOSUPPORT			44
#define	EOPNOTSUPP			45
#define	ENOTSUP				EOPNOTSUPP
#define	EPFNOSUPPORT			46
#define	EAFNOSUPPORT			47
#define	EADDRINUSE			48
#define	EADDRNOTAVAIL			49
#define	ENETDOWN			50
#define	ENETUNREACH			51
#define	ENETRESET			52
#define	ECONNABORTED			53
#define	ECONNRESET			54
#define	ENOBUFS				55
#define	EISCONN				56
#define	ENOTCONN			57
#define	ESHUTDOWN			58
#define	ETOOMANYREFS			59
#define	ETIMEDOUT			60
#define	ECONNREFUSED			61
#define	ELOOP				62
#define	ENAMETOOLONG			63
#define	EHOSTDOWN			64
#define	EHOSTUNREACH			65
#define	ENOTEMPTY			66
#define	EPROCLIM			67
#define	EUSERS				68
#define	EDQUOT				69
#define	ESTALE				70
#define	EREMOTE				71
#define	EBADRPC				72
#define	ERPCMISMATCH			73
#define	EPROGUNAVAIL			74
#define	EPROGMISMATCH			75
#define	EPROCUNAVAIL			76
#define	ENOLCK				77
#define	ENOSYS				78
#define	EFTYPE				79
#define	EAUTH				80
#define	ENEEDAUTH			81
#define	EIDRM				82
#define	ENOMSG				83
#define	EOVERFLOW			84
#define	ECANCELED			85
#define	EILSEQ				86
#define	ENOATTR				87
#define EDOOFUS				88
#define	EBADMSG				89
#define	EMULTIHOP			90
#define	ENOLINK				91
#define	EPROTO				92
#define	ELAST				92


/***********************************************************
 * FCNTL
 ***********************************************************/
#define F_DUPFD				0
#define F_GETFD				1
#define F_SETFD				2
#define F_GETFL				3
#define F_SETFL				4
#define F_GETOWN			5
#define F_SETOWN			6
#define F_GETLK				11
#define F_SETLK				12
#define F_SETLKW			13

#define FD_CLOEXEC			1

#define F_RDLCK				1
#define F_UNLCK				2
#define F_WRLCK				3

struct flock {
	off_t	l_start;
	off_t	l_len;
	pid_t	l_pid;
	short	l_type;
	short	l_whence;
	int	l_sysid;
};


/***********************************************************
 * LSEEK
 ***********************************************************/
#define SEEK_SET			0
#define SEEK_CUR			1
#define SEEK_END			2


/***********************************************************
 * IOCTL - ioctl.h, sockio.h
 ***********************************************************/
#define FIONREAD			0x4004667f

/* XXX
#define SIOCGIFCONF			0x8912
#define SIOCGIFFLAGS			0x8913
#define SIOCGIFBRDADDR			0x8919
#define SIOCGIFNETMASK			0x891B
#define SIOCGIFMTU			0x8921
/ * #define SIOCGIFHWADDR			0x8927 * /
#define SIOCGIFINDEX			0x8933
*/


/***********************************************************
 * MMAP
 ***********************************************************/
#define PROT_NONE			0x00
#define PROT_READ			0x01
#define PROT_WRITE			0x02
#define PROT_EXEC			0x04

#define MAP_FILE			0x0000
#define MAP_SHARED			0x0001
#define MAP_PRIVATE			0x0002
#define MAP_FIXED			0x0010
#define MAP_ANONYMOUS			0x1000
#define MAP_ANON			MAP_ANONYMOUS

#define MAP_FAILED			((void *)-1)

#define MS_SYNC				0

#define MADV_WILLNEED			3


/***********************************************************
 * OPEN
 ***********************************************************/
#define O_ACCMODE			0x0003
#define O_APPEND			0x0008
#define O_CREAT				0x0200
#define O_DIRECT			0x00010000
#define O_EXCL				0x0800
#define O_NOFOLLOW			0x0100
#define O_NDELAY			O_NONBLOCK
#define O_NOCTTY			0x8000
#define O_NONBLOCK			0x0004
#define O_RDONLY			0x0000
#define O_WRONLY			0x0001
#define O_RDWR				0x0002
#define O_SYNC				0x0080
#define O_TRUNC				0x0400


/***********************************************************
 * POLL
 ***********************************************************/
#define POLLIN				0x001
#define POLLPRI				0x002
#define POLLOUT				0x004
#define POLLERR				0x008
#define POLLHUP				0x010
#define POLLNVAL			0x020

typedef unsigned int nfds_t;

struct pollfd {
	int fd;
	short int events;
	short int revents;
};


/***********************************************************
 * SELECT
 ***********************************************************/
#define FD_SETSIZE			1024	/* Max num of fds */


typedef struct _fd_set {
	unsigned char fds_bits[FD_SETSIZE >> 3];
} fd_set;


#define FD_SET(fd, set)		do {				\
	(set)->fds_bits[(fd) >> 3] |= (1 << ((fd) % 8));	\
} while (0)


#define FD_CLR(fd, set)		do {				\
	(set)->fds_bits[(fd) >> 3] &= ~(1 << ((fd) % 8));	\
} while (0)


#define FD_ISSET(fd, set)	((set)->fds_bits[(fd) >> 3] &	\
				(1 << ((fd) % 8)))


#define FD_ZERO(set)		do {				\
	plbzero((set), sizeof(*(set)));				\
} while (0)


/***********************************************************
 * SIGNAL
 ***********************************************************/
#define	SIGHUP				1
#define	SIGINT				2
#define	SIGQUIT				3
#define	SIGILL				4
#define	SIGTRAP				5
#define	SIGABRT				6
#define	SIGIOT				SIGABRT
#define	SIGEMT				7
#define	SIGFPE				8
#define	SIGKILL				9
#define	SIGBUS				10
#define	SIGSEGV				11
#define	SIGSYS				12
#define	SIGPIPE				13
#define	SIGALRM				14
#define	SIGTERM				15
#define	SIGURG				16
#define	SIGSTOP				17
#define	SIGTSTP				18
#define	SIGCONT				19
#define	SIGCHLD				20
#define	SIGTTIN				21
#define	SIGTTOU				22
#define	SIGIO				23
#define	SIGXCPU				24
#define	SIGXFSZ				25
#define	SIGVTALRM			26
#define	SIGPROF				27
#define	SIGWINCH			28
#define	SIGINFO				29
#define	SIGUSR1				30
#define	SIGUSR2				31

#define	SIGTHR				32
#define	SIGLWP				SIGTHR

#define	SIGRTMIN			65
#define	SIGRTMAX			126

#define NSIG				128


/* XXX not in freebsd
/ * The following two values are from kernel 2.6.20-16 * /
#if 0
#define SIGRTMIN			34
#define SIGCANCEL			34
#define SIGSETXID			35

#endif
*/

#define _PL_BITS			(4 << 3)

#define PL_SIGSET_VAL(signo)		(1 << (((signo)-1) % _PL_BITS))
#define PL_SIGSET_IDX(signo)		(((signo)-1) >> 5)
//XXX #define PL_SIGSET_IDX(signo)		(((signo)-1) / _PL_BITS)


/*
 * sigset_t
 */
typedef struct {
	uint32	set[_PL_BITS >> 3];	/* max 128 signals */
} sigset_t;

/*
 * sigval_t
 */
typedef union sigval {
	int	 sival_int;
	void	*sival_ptr;
} sigval_t;

/*
 * siginfo_t
 */
typedef struct siginfo {
	int		 si_signo;
	int		 si_errno;
	int		 si_code;
	pid_t		 si_pid;
	uid_t		 si_uid;
	int		 si_status;
	void		*si_addr;
	union sigval	 si_value;
	long 		 si_band;
} siginfo_t;

/*
 * jmp_buf
 */
#define PLJMPBUF_STORAGE		12
typedef struct _jmp_buf {
        long jmp_buf[PLJMPBUF_STORAGE];
} jmp_buf;

/*
 * sigjmp_buf
 */
typedef struct _sigjmp_buf {
        long jmp_buf[PLJMPBUF_STORAGE];
} sigjmp_buf;

/*
 * How to treat signal sets.
 */
#define SIG_BLOCK			1
#define SIG_UNBLOCK			2
#define SIG_SETMASK			3

/*
 * Default signal handlers.
 */
#define SIG_ERR				((void *)-1)
#define SIG_DFL				((void *) 0)
#define SIG_IGN				((void *) 1)

/* 
 * Sigaltstack
 */
#define SS_ONSTACK			1
#define SS_DISABLE			2

#define MINSIGSTKSZ			2048
#define SIGSTKSZ			8192

typedef struct sigaltstack {
	void	*ss_sp;
	size_t	 ss_size;
	int	 ss_flags;
} stack_t;

/*
 * Sigaction
 */
struct sigaction {
	union {
		void	(*_sa_handler)(int);
		void	(*_sa_sigaction)(int, struct siginfo *, void *);
	} _u;
	unsigned long	sa_flags;
	sigset_t	sa_mask;
//XXX	void		(*sa_restorer)(void);
};

/* no kernel sigaction in freebsd
struct kernel_sigaction {
	void		(*k_sa_handler)(int);
	unsigned long	sa_flags;
	void		(*sa_restorer)(void);
	sigset_t	sa_mask;
};
*/

#define sa_handler	_u._sa_handler
#define sa_sigaction	_u._sa_sigaction


/*
 * sa_flags values:
 *
 * SA_ONSTACK indicates that a registered stack_t will be used.
 * SA_RESTART flag to get restarting signals (which were the default long ago)
 * SA_NOCLDSTOP flag to turn off SIGCHLD when children stop.
 * SA_RESETHAND clears the handler when the signal is delivered.
 * SA_NOCLDWAIT flag on SIGCHLD to inhibit zombies.
 * SA_NODEFER prevents the current signal from being masked in the handler.
 *
 * SA_ONESHOT and SA_NOMASK are the historical Linux names for the Single
 * Unix names RESETHAND and NODEFER respectively.
 */
#define	SA_NOCLDSTOP			0x0008
#define	SA_NOCLDWAIT			0x0020
#define	SA_SIGINFO			0x0040
#define	SA_ONSTACK			0x0001
#define	SA_RESTART			0x0002
#define	SA_NODEFER			0x0010
#define	SA_RESETHAND			0x0004

/* XXX not in freebsd
#define SA_NOMASK			SA_NODEFER
#define SA_ONESHOT			SA_RESETHAND
#define SA_RESTORER			0x04000000
*/


/***********************************************************
 * SOCKET
 ***********************************************************/
/*
 * Socket Address Family.
 */
#define AF_UNSPEC			0
#define AF_UNIX				1
#define AF_LOCAL			1
#define AF_INET				2
#define	AF_IMPLINK			3
#define	AF_PUP				4
#define	AF_CHAOS			5
#define	AF_NETBIOS			6
#define	AF_ISO				7
#define	AF_OSI				AF_ISO
#define	AF_ECMA				8
#define	AF_DATAKIT			9
#define	AF_CCITT			10
#define	AF_SNA				11
#define AF_DECnet			12
#define AF_DLI				13
#define AF_LAT				14
#define	AF_HYLINK			15
#define	AF_APPLETALK			16
#define	AF_ROUTE			17
#define	AF_LINK				18
#define	pseudo_AF_XTP			19
#define	AF_COIP				20
#define	AF_CNT				21
#define pseudo_AF_RTIP			22
#define	AF_IPX				23
#define	AF_SIP				24
#define	pseudo_AF_PIP			25
#define	AF_ISDN				26
#define	AF_E164				AF_ISDN
#define	pseudo_AF_KEY			27
#define	AF_INET6			28
#define	AF_NATM				29
#define	AF_ATM				30
#define pseudo_AF_HDRCMPLT		31
#define	AF_NETGRAPH			32
#define	AF_SLOW				33
#define	AF_SCLUSTER			34
#define	AF_ARP				35
#define	AF_BLUETOOTH			36
#define	AF_IEEE80211			37
#define	AF_MAX				38

/*
 * Protocol Family.
 */
#define PF_UNSPEC			AF_UNSPEC
#define PF_LOCAL			AF_LOCAL
#define PF_UNIX				AF_LOCAL

/*
 * Socket Types.
 */
#define	SOCK_STREAM			1
#define	SOCK_DGRAM			2
#define	SOCK_RAW			3
#define	SOCK_RDM			4
#define SOCK_SEQPACKET			5

typedef uint32	socklen_t;
typedef uint8	sa_family_t;

struct sockaddr {
	uint8		sa_len;
	sa_family_t	sa_family;
	char		sa_data[14];
};

struct linger {
	int		l_onoff;
	int		l_linger;
};

/*
 * IPv4
 */
typedef uint32	in_addr_t;
typedef uint16	in_port_t;

struct in_addr {
	in_addr_t	s_addr;
};

struct sockaddr_in {
	uint8		sin_len;
	sa_family_t	sin_family;

	in_port_t	sin_port;
	struct in_addr	sin_addr;

	char		sin_zero[8];
};

/* Correct? */
struct ip_mreq {
	struct in_addr	imr_multiaddr;
	struct in_addr	imr_interface;
};

struct ip_mreqn {
	struct in_addr	imr_multiaddr;
	struct in_addr	imr_address;
	int		imr_ifindex;
};


/*
 * IPv6
 */
struct in6_addr {
	union {
		uint8	u6_addr8[16];
		uint16	u6_addr16[8];
		uint32	u6_addr32[4];
	} u6_addr;
};

#define s6_addr		u6_addr.u6_addr8
#define s6_addr16	u6_addr.u6_addr16
#define s6_addr32	u6_addr.u6_addr32

struct sockaddr_in6 {
	uint8		sin6_len;
	uint8		sin6_family;
	uint16		sin6_port;
	uint32		sin6_flowinfo;
	struct in6_addr	sin6_addr;
	uint32		sin6_scope_id;
};

struct ipv6_mreq {
	struct in6_addr	ipv6mr_multiaddr;
	unsigned int	ipv6mr_interface;
};


/*
 * Argument 'how' for plsock_shutdown().
 */
#define SHUT_RD				0
#define SHUT_WR				1
#define SHUT_RDWR 			2

/*
 * Socket Levels plsys_getsockopt() and plsys_setsockopt().
 */
#define SOL_SOCKET			0xffff

/* XXX where are they?
#define SOL_IP				0
#define SOL_TCP				6
#define SOL_UDP				17
#define SOL_IPV6			41
#define SOL_ICMPV6			58
#define SOL_SCTP			132
#define SOL_UDPLITE			136
#define SOL_RAW				255
#define SOL_IPX				256
#define SOL_AX25			257
#define SOL_ATALK			258
#define SOL_NETROM			259
#define SOL_ROSE			260
#define SOL_DECNET			261
#define SOL_X25				262
#define SOL_PACKET			263
#define SOL_ATM				264
#define SOL_AAL				265
#define SOL_IRDA			266
#define SOL_NETBEUI			267
#define SOL_LLC				268
#define SOL_DCCP			269
#define SOL_NETLINK			270
#define SOL_TIPC			271
*/

/*
 * Socket Options for SOL_SOCKET level.
 */
#define	SO_DEBUG			0x0001
#define	SO_ACCEPTCONN			0x0002
#define	SO_REUSEADDR			0x0004
#define	SO_KEEPALIVE			0x0008
#define	SO_DONTROUTE			0x0010
#define	SO_BROADCAST			0x0020
#define	SO_USELOOPBACK			0x0040
#define	SO_LINGER			0x0080
#define	SO_OOBINLINE			0x0100
#define	SO_REUSEPORT			0x0200
#define	SO_TIMESTAMP			0x0400
#define	SO_NOSIGPIPE			0x0800
#define	SO_ACCEPTFILTER			0x1000
#define	SO_BINTIME			0x2000
#define	SO_NO_OFFLOAD			0x4000
#define	SO_NO_DDP			0x8000
#define	SO_SNDBUF			0x1001
#define	SO_RCVBUF			0x1002
#define	SO_SNDLOWAT			0x1003
#define	SO_RCVLOWAT			0x1004
#define	SO_SNDTIMEO			0x1005
#define	SO_RCVTIMEO			0x1006
#define	SO_ERROR			0x1007
#define	SO_TYPE				0x1008
#define	SO_LABEL			0x1009
#define	SO_PEERLABEL			0x1010
#define	SO_LISTENQLIMIT			0x1011
#define	SO_LISTENQLEN			0x1012
#define	SO_LISTENINCQLEN		0x1013
#define	SO_SETFIB			0x1014

/*
 * Socket options for IPPROTO_IP level.
 */
#define	IP_OPTIONS			1
#define	IP_HDRINCL			2
#define	IP_TOS				3
#define	IP_TTL				4
#define	IP_RECVOPTS			5
#define	IP_RECVRETOPTS			6
#define	IP_RECVDSTADDR			7
#define	IP_SENDSRCADDR			IP_RECVDSTADDR
#define	IP_RETOPTS			8
#define	IP_MULTICAST_IF			9
#define	IP_MULTICAST_TTL		10
#define	IP_MULTICAST_LOOP		11
#define	IP_ADD_MEMBERSHIP		12
#define	IP_DROP_MEMBERSHIP		13
#define	IP_MULTICAST_VIF		14
#define	IP_RSVP_ON			15
#define	IP_RSVP_OFF			16
#define	IP_RSVP_VIF_ON			17
#define	IP_RSVP_VIF_OFF			18
#define	IP_PORTRANGE			19
#define	IP_RECVIF			20
#define	IP_IPSEC_POLICY			21
#define	IP_FAITH			22
#define	IP_ONESBCAST			23
#define	IP_FW_TABLE_ADD			40
#define	IP_FW_TABLE_DEL			41
#define	IP_FW_TABLE_FLUSH		42
#define	IP_FW_TABLE_GETSIZE		43
#define	IP_FW_TABLE_LIST		44
#define	IP_FW_ADD			50
#define	IP_FW_DEL			51
#define	IP_FW_FLUSH			52
#define	IP_FW_ZERO			53
#define	IP_FW_GET			54
#define	IP_FW_RESETLOG			55
#define IP_FW_NAT_CFG			56
#define IP_FW_NAT_DEL			57
#define IP_FW_NAT_GET_CONFIG		58
#define IP_FW_NAT_GET_LOG		59
#define	IP_DUMMYNET_CONFIGURE		60
#define	IP_DUMMYNET_DEL			61
#define	IP_DUMMYNET_FLUSH		62
#define	IP_DUMMYNET_GET			64
#define	IP_RECVTTL			65
#define	IP_MINTTL			66
#define	IP_DONTFRAG			67
#define	IP_ADD_SOURCE_MEMBERSHIP	70
#define	IP_DROP_SOURCE_MEMBERSHIP	71
#define	IP_BLOCK_SOURCE			72
#define	IP_UNBLOCK_SOURCE		73
#define	IP_MSFILTER			74

#define	MCAST_JOIN_GROUP		80
#define	MCAST_LEAVE_GROUP		81
#define	MCAST_JOIN_SOURCE_GROUP		82
#define	MCAST_LEAVE_SOURCE_GROUP	83
#define	MCAST_BLOCK_SOURCE		84
#define	MCAST_UNBLOCK_SOURCE		85

#define	IP_DEFAULT_MULTICAST_TTL	1
#define	IP_DEFAULT_MULTICAST_LOOP	1

#define	IP_MIN_MEMBERSHIPS		31
#define	IP_MAX_MEMBERSHIPS		4095
#define	IP_MAX_SOURCE_FILTER		1024

/*
 * Socket options for IPPROTO_TCP level.
 */
#define	TCP_NODELAY			0x01
#define	TCP_MAXSEG			0x02
#define TCP_NOPUSH			0x04
#define TCP_NOOPT			0x08
#define TCP_MD5SIG			0x10
#define	TCP_INFO			0x20
#define	TCP_CONGESTION			0x40
#define	TCP_CA_NAME_MAX			16

/*
 * Socket options for IPPROTO_IPV6 level.
 */
#define IPV6_SOCKOPT_RESERVED1		3
#define IPV6_UNICAST_HOPS		4
#define IPV6_MULTICAST_IF		9
#define IPV6_MULTICAST_HOPS		10
#define IPV6_MULTICAST_LOOP		11
#define IPV6_JOIN_GROUP			12
#define IPV6_LEAVE_GROUP		13
#define IPV6_PORTRANGE			14
#define ICMP6_FILTER			18
#define IPV6_2292PKTINFO		19
#define IPV6_2292HOPLIMIT		20
#define IPV6_2292NEXTHOP		21
#define IPV6_2292HOPOPTS		22
#define IPV6_2292DSTOPTS		23
#define IPV6_2292RTHDR			24
#define IPV6_2292PKTOPTIONS		25
#define IPV6_CHECKSUM			26
#define IPV6_V6ONLY			27
#define IPV6_BINDV6ONLY			IPV6_V6ONLY
#define IPV6_IPSEC_POLICY		28
#define IPV6_FAITH			29
#define IPV6_FW_ADD			30
#define IPV6_FW_DEL			31
#define IPV6_FW_FLUSH			32
#define IPV6_FW_ZERO			33
#define IPV6_FW_GET			34
#define IPV6_RTHDRDSTOPTS		35
#define IPV6_RECVPKTINFO		36
#define IPV6_RECVHOPLIMIT		37
#define IPV6_RECVRTHDR			38
#define IPV6_RECVHOPOPTS		39
#define IPV6_RECVDSTOPTS		40
#define IPV6_RECVRTHDRDSTOPTS		41
#define IPV6_USE_MIN_MTU		42
#define IPV6_RECVPATHMTU		43
#define IPV6_PATHMTU			44
#if 0
#define IPV6_REACHCONF			45	/* obsoleted during 2292bis -> 3542 */
#endif
#define IPV6_PKTINFO			46
#define IPV6_HOPLIMIT			47
#define IPV6_NEXTHOP			48
#define IPV6_HOPOPTS			49
#define IPV6_DSTOPTS			50
#define IPV6_RTHDR			51
#if 0
#define IPV6_PKTOPTIONS			52	/* obsoleted by RFC3542 */
#endif
#define IPV6_RECVTCLASS			57
#define IPV6_AUTOFLOWLABEL		59
#define IPV6_TCLASS			61
#define IPV6_DONTFRAG			62
#define IPV6_PREFER_TEMPADDR		63

#define IPV6_DEFAULT_MULTICAST_HOPS	1
#define IPV6_DEFAULT_MULTICAST_LOOP	1

/*
 * Flags for plsys_send() and plsys_recv().
 */
#define MSG_OOB				0x1
#define MSG_PEEK			0x2
#define MSG_DONTROUTE			0x4
#define MSG_EOR				0x8
#define MSG_TRUNC			0x10
#define MSG_CTRUNC			0x20
#define MSG_WAITALL			0x40
#define MSG_NOTIFICATION		0x2000
#define MSG_DONTWAIT			0x80
#define MSG_EOF				0x100
#define MSG_NBIO			0x4000
#define MSG_COMPAT			0x8000
#define MSG_SOCALLBCK			0x10000
#define MSG_NOSIGNAL			0x20000

/*
 * struct msghdr
 */
struct msghdr {
	void		*msg_name;
	socklen_t	 msg_namelen;
	struct iovec	*msg_iov;
	int		 msg_iovlen;
	void		*msg_control;
	socklen_t	 msg_controllen;
	int		 msg_flags;
};


/***********************************************************
 * INTERFACES - net/if.h
 ***********************************************************/
#define IFNAMSIZ			16

#define	IFF_UP				0x1
#define	IFF_BROADCAST			0x2
#define	IFF_DEBUG			0x4
#define	IFF_LOOPBACK			0x8
#define	IFF_POINTOPOINT			0x10
#define	IFF_SMART			0x20
#define	IFF_DRV_RUNNING			0x40
#define	IFF_NOARP			0x80
#define	IFF_PROMISC			0x100
#define	IFF_ALLMULTI			0x200
#define	IFF_DRV_OACTIVE			0x400
#define	IFF_SIMPLEX			0x800
#define	IFF_LINK0			0x1000
#define	IFF_LINK1			0x2000
#define	IFF_LINK2			0x4000
#define	IFF_ALTPHYS			IFF_LINK2
#define	IFF_MULTICAST			0x8000
#define	IFF_PPROMISC			0x20000
#define	IFF_MONITOR			0x40000
#define	IFF_STATICARP			0x80000
#define	IFF_NEEDSGIANT			0x100000

struct ifreq {
	char	ifr_name[IFNAMSIZ];
	union {
		struct sockaddr	ifru_addr;
		struct sockaddr	ifru_dstaddr;
		struct sockaddr	ifru_broadaddr;
		short 		ifru_flags[2];
		short		ifru_index;
		int 		ifru_metric;
		int		ifru_mtu;
		int		ifru_phys;
		int		ifru_media;
		caddr_t		ifru_data;
		int		ifru_cap[2];
	} ifr_ifru;
#define ifr_addr	ifr_ifru.ifru_addr
#define ifr_dstaddr	ifr_ifru.ifru_dstaddr
#define ifr_broadaddr	ifr_ifru.ifru_broadaddr
#define ifr_flags	ifr_ifru.ifru_flags[0]
#define ifr_flagshigh	ifr_ifru.ifru_flags[1]
#define ifr_metric	ifr_ifru.ifru_metric
#define ifr_mtu		ifr_ifru.ifru_mtu
#define ifr_phys	ifr_ifru.ifru_phys
#define ifr_media	ifr_ifru.ifru_media
#define ifr_data	ifr_ifru.ifru_data
#define ifr_regcap	ifr_ifru.ifru_cap[0]
#define ifr_curcap	ifr_ifru.ifru_cap[1]
#define ifr_index	ifr_ifru.ifru_index
};

struct ifconf {
	int	ifc_len;
	union {
		caddr_t	ifcu_buf;
		struct	ifreq *ifcu_req;
	} ifc_ifcu;
#define ifc_buf		ifc_ifcu.ifcu_buf
#define ifc_req		ifc_ifcu.ifcu_req
};


/***********************************************************
 * netdb.h
 ***********************************************************/
struct addrinfo {
	int		 ai_flags;
	int		 ai_family;
	int		 ai_socktype;
	int		 ai_protocol;
	socklen_t	 ai_addrlen;
	char		*ai_canonname;
	struct sockaddr	*ai_addr;
	struct addrinfo *ai_next;
};

struct hostent {
	char	 *h_name;
	char	**h_aliases;
	int	  h_addrtype;
	int	  h_length;
	char	**h_addr_list;
#define h_addr h_addr_list[0]
};

struct protoent {
	char	 *p_name;
	char	**p_aliases;
	int	  p_proto;
};

#define	NETDB_INTERNAL		-1
#define	NETDB_SUCCESS		0
#define	HOST_NOT_FOUND		1
#define	TRY_AGAIN		2
#define	NO_RECOVERY		3
#define	NO_DATA			4
#define	NO_ADDRESS		NO_DATA

#define AI_PASSIVE		1
#define AI_CANONNAME		2
#define AI_NUMERICHOST		4
#define AI_NUMERICSERV		8

#define NI_NUMERICHOST		2
#define NI_NAMEREQD		4
#define NI_NUMERICSERV		8
#define NI_DGRAM		16
#define NI_MAXHOST		1025
#define NI_MAXSERV		32


/***********************************************************
 * STAT
 ***********************************************************/
#define	S_ISUID			0004000
#define	S_ISGID			0002000
#define	S_ISTXT			0001000
#define	S_IRWXU			0000700
#define	S_IRUSR			0000400
#define	S_IWUSR			0000200
#define	S_IXUSR			0000100
#define	S_IREAD			S_IRUSR
#define	S_IWRITE		S_IWUSR
#define	S_IEXEC			S_IXUSR
#define	S_IRWXG			0000070
#define	S_IRGRP			0000040
#define	S_IWGRP			0000020
#define	S_IXGRP			0000010
#define	S_IRWXO			0000007
#define	S_IROTH			0000004
#define	S_IWOTH			0000002
#define	S_IXOTH			0000001

#define	S_IFMT	 		0170000
#define	S_IFIFO	 		0010000
#define	S_IFCHR	 		0020000
#define	S_IFDIR	 		0040000
#define	S_IFBLK	 		0060000
#define	S_IFREG	 		0100000
#define	S_IFLNK	 		0120000
#define	S_IFSOCK 		0140000
#define	S_ISVTX	 		0001000
#define	S_IFWHT  		0160000

#define S_BLKSIZE		512

typedef struct stat {
	dev_t		st_dev;
	ino_t		st_ino;
	mode_t		st_mode;
	nlink_t		st_nlink;
	uid_t		st_uid;
	gid_t		st_gid;
	dev_t		st_rdev;

	time_t		st_atime;
	long		st_atime_nsec;
	time_t		st_mtime;
	long		st_mtime_nsec;
	time_t		st_ctime;
	long		st_ctime_nsec;

	off_t		st_size;
	uint64		st_blocks;
	uint32		st_blksize;
	uint32		st_flags;
	uint32		st_gen;
	int32		st_lspare;

	time_t		st_birthtime;
	long		st_bithtimensec;

	/* PAD */
        unsigned int :(8 / 2) * (16 - (int)8 /* sizeof(struct __timespec) */ );
        unsigned int :(8 / 2) * (16 - (int)8 /* sizeof(struct __timespec) */ );
} stat_t;

#define	S_ISDIR(m)	(((m) & 0170000) == 0040000)
#define	S_ISCHR(m)	(((m) & 0170000) == 0020000)
#define	S_ISBLK(m)	(((m) & 0170000) == 0060000)
#define	S_ISREG(m)	(((m) & 0170000) == 0100000)
#define	S_ISFIFO(m)	(((m) & 0170000) == 0010000)
#define	S_ISLNK(m)	(((m) & 0170000) == 0120000)
#define	S_ISSOCK(m)	(((m) & 0170000) == 0140000)
#define	S_ISWHT(m)	(((m) & 0170000) == 0160000)


/***********************************************************
 * TIME
 ***********************************************************/
struct timespec {
	time_t	tv_sec;
	long	tv_nsec;
};

struct timeval {
	time_t	tv_sec;
	long	tv_usec;
};

struct timezone {
	int	tz_minuteswest;
	int	tz_dsttime;
};

struct tm {
	int	 tm_sec;
	int	 tm_min;
	int	 tm_hour;
	int	 tm_mday;
	int	 tm_mon;
	int	 tm_year;
	int	 tm_wday;
	int	 tm_yday;
	int	 tm_isdst;

	long int tm_gmtoff;
	char	*tm_zone;
};


/***********************************************************
 * UNAME
 ***********************************************************/
struct utsname {
	char sysname[256];
	char nodename[256];
	char release[256];
	char version[256];
	char machine[256];
};


/***********************************************************
 * WAIT
 ***********************************************************/
#define WNOHANG				0x00000001
#define WUNTRACED			0x00000002
#define WSTOPPED			WUNTRACED
#define WCONTINUED			0x00000004
#define WNOWAIT				0x00000008

#define WTERMSIG(status)		((status) & 0x7f)

#define WIFEXITED(status)		(WTERMSIG((status)) == 0)

#define WEXITSTATUS(status)		((status) >> 8)

#define WIFSIGNALED(status)		(WTERMSIG(status) != 0x7f && \
					WTERMSIG(status) != 0)


#endif	/* FREEBSD_I386_H */
