/*
 * Copyright 2007-2010 Claudio Castiglia
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/**
 * @file src/lib/plc/plsigblock.c
 */
#include <plc.h>
#include <pllog.h>


void PLEXPORT
plsigblock(sigset_t *oset)
{
	PLSTATUS st;
	sigset_t set;

	plsys_sigfillset(&set);
	st = plsys_tsigmask_nt(SIG_BLOCK, &set, oset);
	if (st != PLSTATUS_SUCCESS) {
		pldebug("%s: Can't block signals, peludo code=%s, errno=%d\n",
		    __FUNCTION__,
		    plstrstatus(st),
		    errno);

		plsys_exit(-1);   /* XXX Let the user to manage the error? */
		/* NOTREACHED */
	}
}


void PLEXPORT
plsigunblock(const sigset_t *set)
{
	PLSTATUS st;

	st = plsys_tsigmask_nt(SIG_SETMASK, set, NULL);
	if (st != PLSTATUS_SUCCESS) {
		pldebug("%s: Can't restore signals, peludo code=%s, errno=%d\n",
		    __FUNCTION__,
		    plstrstatus(st),
		    errno);

		plsys_exit(-1);   /* XXX Let the user to manage the error? */
		/* NOTREACHED */
	}
}


