/*
 * Copyright 2007-2010 Claudio Castiglia
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/**
 * @file src/lib/plc/std/plstd_init.c
 */
#include <plc.h>
#include <stdio.h>
#include <unistd.h>


#ifdef PLOS_win32
/* XXX */
#error "plstd_init.c NOT PORTED TO WIN32."
#endif


/*
 * Initialized to 0 due to kernel coding restritions
 * (can't assume PLC is not integrated into the peludo kernel);
 * see src/core/kernel/README.
 */
static FILE _stdin  = { 0 };
static FILE _stdout = { 0 };
static FILE _stderr = { 0 };

FILE PLEXPORT *stdin  = NULL;
FILE PLEXPORT *stdout = NULL;
FILE PLEXPORT *stderr = NULL;


/**
 * Initialize the standard C part of PLC.
 *
 * @return A Peludo Status code is returned.
 *
 * @see PLSTATUS
 */
PLSTATUS PLEXPORT
plstd_init()
{
	_stdin.f_fd  = STDIN_FILENO;
	_stdout.f_fd = STDOUT_FILENO;
	_stderr.f_fd = STDERR_FILENO;

	stdin  = &_stdin;
	stdout = &_stdout;
	stderr = &_stderr;

	return PLSTATUS_SUCCESS;
}

