/*
 * Copyright 2007-2010 Claudio Castiglia
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/**
 * @file src/lib/plc/std/stdio_fopen.c
 */
#include <plc.h>
#include <stdio.h>
#include <string.h>


struct _mode2flags {
	char	*mode;
	int	 flags;
	int	 cmode;
};


static struct _mode2flags *
_parsemode(const char *mode)
{
	/* Dont care about "b" */
	static struct _mode2flags mode2flags[] = {
		{ "r",	O_RDONLY,	0			},
		{ "r+", O_RDWR,		0			},
		{ "w",	O_WRONLY,	O_CREAT | O_TRUNC	},
		{ "w+",	O_RDWR,		O_CREAT | O_TRUNC	},
		{ "a",	O_WRONLY,	O_CREAT | O_APPEND	},
		{ "a+",	O_RDWR,		O_CREAT | O_APPEND	},
		{ NULL, 0, 0 }
	};
	struct _mode2flags *p;

	for (p = mode2flags; p->mode != NULL; p++) {
		if (!strcmp(mode, p->mode)) {
			return p;
		}
	}

	return NULL;
}


static FILE *
_fdopen(long fd, const char *mode)
{
	FILE *f;

	if ((f = plmalloc(sizeof(FILE))) == NULL) {
		return NULL;
	}

	plbzero((uint8 *)f, sizeof(FILE));

	f->f_fd = fd;

	return f;
}


FILE PLEXPORT *
fopen(const char *path, const char *mode)
{
	struct _mode2flags *of;
	FILE *f;
	long fd;

	if (path == NULL || mode == NULL || (of = _parsemode(mode)) == NULL) {
		errno = EINVAL;
		return NULL;
	}

	if (plsys_open(&fd, path, of->flags, of->cmode) != PLSTATUS_SUCCESS) {
		return NULL;
	}

	f = _fdopen(fd, mode);
	if (f == NULL) {
		plsys_close(fd);
	}

	return f;
}


FILE PLEXPORT *
fdopen(int fd, const char *mode)
{
	if (mode == NULL || fd < 0) {
		errno = EINVAL;
		return NULL;
	}

	return _fdopen(fd, mode);
}

