/*
 * Copyright 2007-2010 Claudio Castiglia
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/**
 * @file src/lib/plc/std/stdio_vfprintf.c
 */
#include <plc.h>
#include <limits.h>
#include <stdio.h>


extern int plvnprint_cb(const char *fmt, va_list args,
    int (*doprint)(void *, const char *, size_t), void *par, size_t siz);


static int
_print(void *dst, const char *src, size_t siz)
{
	size_t ret, psiz;
	FILE *f = (FILE *)dst;

	psiz = plstrlen(src);
	if (psiz > siz) {
		psiz = siz;
	}

	plsys_write((long)(f->f_fd), src, psiz, &ret);

	return ret;
}


int PLEXPORT
vfprintf(FILE *f, const char *fmt, va_list ap)
{
	if (f == NULL) {
		errno = EINVAL;
		return -1;
	}

	return plvnprint_cb(fmt, ap, (void *)_print, f, INT_MAX);
}


