/*
 * Copyright 2007-2010 Claudio Castiglia
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
#include <unistd.h>


/**
 * Force all currently queued I/O operations associated with the file indicated
 * by file descriptor fildes to the synchronised I/O completion state.
 * <b>This function is a cancellation point.<b>
 *
 * @param fd (IN) File descriptor.
 *
 * @return On success <code>0</code> is returned; otherwise, <code>-1</code>
 * and <code>errno</code> is set properly.
 *
 * @see http://opengroup.org/onlinepubs/007908775/xsh/fdatasync.html
 *
 * @see plsys_fdatasync()
 */
int PLEXPORT
fdatasync(int fd)
{
	PLSTATUS st;

	plenter_cancel_point();

	st = plsys_fdatasync((long)fd);

	plleave_cancel_point();

	return ((st == PLSTATUS_SUCCESS) ? 0 : -1);
}


