/*
 * Copyright 2007-2010 Claudio Castiglia
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/**
 * @file src/lib/plc/sysdep/freebsd/i386/_plsys_palloc.c
 */
#include <plc.h>


PLSTATUS PLEXPORT
_plsys_palloc(long *err, long *addr, unsigned long size, PLMFLAGS flags)
{
	long val, error;

	if (addr == NULL) {
		return PLSTATUS_INVALID;
	}

	/*
	 * caddr_t * mmap(caddr_t addr, size_t len, int prot,
	 *                int flags, int fd, off_t pos);
	 *
	 * Notice that off_t is a 64 bit value.
	 */
	val = _plsyscall(&error, SYS_mmap, 0, size, plsys_plmflags2prot(flags),
	    MAP_PRIVATE | MAP_ANONYMOUS, -1, (off_t)0);
	if (val != -1) {
		*addr = val;
		return PLSTATUS_SUCCESS;
	}

	if (err != NULL)  {
		*err = error;
	}

	return ((error == EAGAIN) ?  PLSTATUS_TRY_AGAIN : PLSTATUS_FAILED);
}


