/*
 * Copyright 2007-2010 Claudio Castiglia
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/**
 * @file src/lib/plc/sysdep/freebsd/i386/plsys_tsigmask.c
 */
#include <plc.h>


PLSTATUS PLEXPORT
plsys_tsigmask_nt(int how, const sigset_t *newset, sigset_t *oldset)
{
	if (plsyscall(SYS_sigprocmask, how, newset, oldset, NSIG / 8)
	    != -1) {
		return PLSTATUS_SUCCESS;
	}

	return ((errno == EAGAIN) ? PLSTATUS_TRY_AGAIN : PLSTATUS_FAILED);
}


PLSTATUS PLEXPORT
plsys_tsigmask(int how, const sigset_t *newset, sigset_t *oldset)
{
	sigset_t realset;

	if (how != SIG_BLOCK && how != SIG_UNBLOCK && how != SIG_SETMASK) {
		return PLSTATUS_INVALID;
	}

	/*
	 * SIGCANCEL and SIGSETXID should not be blocked.
	 */
	if (newset != NULL && (how == SIG_BLOCK || how == SIG_SETMASK)) {
		plsys_memmove(&realset, newset, sizeof(realset));

/* XXX
		plsys_sigdelset(&realset, SIGCANCEL);
		plsys_sigdelset(&realset, SIGSETXID);
*/

		newset = &realset;
	}

	return plsys_tsigmask_nt(how, newset, oldset);
}


