/*
 * Copyright 2007-2010 Claudio Castiglia
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/**
 * @file src/lib/plc/sysdep/freebsd/i386/plsys_tsigwait.c
 */
#include <plc.h>


PLSTATUS PLEXPORT
plsys_tsigwait(const sigset_t *set, int *signo)
{
	return PLSTATUS_NOT_IMPLEMENTED;
#if 0
	sigset_t realset;
	int signal;

	/*
	 * SIGCANCEL and SIGSETXID should not be blocked.
	 */
	if (set != NULL) {
		plsys_memmove(&realset, set, sizeof(realset));

		plsys_sigaddset(&realset, SIGCANCEL);
		plsys_sigaddset(&realset, SIGSETXID);

		set = &realset;
	}

	/*
	 * long sys_rt_sigtimedwait(
	 *		const sigset_t __user *uthese,
	 *		siginfo_t __user *uinfo,
	 *		const struct timespec __user *uts,
	 *		size_t sigsetsize
	 * );
	 *
	 * rt_sigtimedwait synchronously waits for queued signals specified in
	 * uthese. The uinfo value, if non-NULL, indicates to the call to
	 * return a structure of type siginfo_t containing information about
	 * the signal. The uts variable enables an upper bound on the time for
	 * when the process is suspended. sigsetsize should indicate the size
	 * of a sigset_t type.
	 */

	signal = plsyscall(SYS_rt_sigtimedwait, set, NULL, NULL, NSIG / 8, 0,
	    0);

	if (signal > 0) {
		if (signal == SIGCANCEL || signal == SIGSETXID) {
			/*
 			 * XXX
 			 * Because of the way sys_rt_sigtimedwait() works
 			 * if there exists a SIGCANCEL handler it was not
 			 * called and the thread was not terminated.
 			 * So we finish it here.
 			 */
			plexit(-1);
			/* NOTREACHED */
		}

		if (signo != NULL) {
			*signo = signal;
		}

		return PLSTATUS_SUCCESS;
	}

	return ((errno == EAGAIN) ? PLSTATUS_TRY_AGAIN : PLSTATUS_FAILED);
#endif
}


