/*
 * Copyright 2007-2010 Claudio Castiglia
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/**
 * @file src/lib/plc/sysdep/linux/i386/_plsyscall.S
 */
.text
.global _plsyscall


#define ARG_ERR		0x08
#define ARG_SYS		0x0C
#define ARG_1		0x10
#define ARG_2		0x14
#define ARG_3		0x18
#define ARG_4		0x1C
#define ARG_5		0x20
#define ARG_6		0x24


/*
 * long _plsyscall(long *errno, long sysno,
 *                 long a1, long a2, long a3, long a4, long a5, long a6)
 *
 * On Linux/i386 this function requires 6 arguments,
 * no matter whether the syscall called uses them or not.
 */
_plsyscall:	pushl	%ebp
		movl	%esp, %ebp
		pushl	%ebx
		pushl	%ecx
		pushl	%esi
		pushl	%edi
		pushl	%ebp

		movl	ARG_SYS(%ebp), %eax
		movl	ARG_1(%ebp), %ebx
		movl	ARG_2(%ebp), %ecx
		movl	ARG_3(%ebp), %edx
		movl	ARG_4(%ebp), %esi
		movl	ARG_5(%ebp), %edi
		movl	ARG_6(%ebp), %ebp
		int 	$0x80

		popl	%ebp
		cmpl	$-4095, %eax
		jb	_plsyscall_end

		negl	%eax

		movl	ARG_ERR(%ebp), %ebx
		jz	_plsyscall_err
		movl	%eax, (%ebx)

_plsyscall_err:	xorl	%eax, %eax
		decl	%eax

_plsyscall_end:	popl	%edi
		popl	%esi
		popl	%ecx	
		popl	%ebx
		popl	%ebp
		ret

