/*
 * Copyright 2007-2010 Claudio Castiglia
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/**
 * @file src/lib/plc/sysdep/linux/i386/plsock_sendto.c
 */
#include <plc.h>
#include <plsock.h>


PLSTATUS PLEXPORT
plsock_sendto(size_t *sent, long sock, const void *buf, size_t len, int flags,
    const struct sockaddr *to, socklen_t tolen)
{
	long args[5], ret;

	args[0] = sock;
	args[1] = (long)buf;
	args[2] = (long)len;
	args[3] = (long)flags;
	args[4] = (long)to;
	args[5] = (long)tolen;

	ret = plsyscall(SYS_socketcall, SOCKETCALL_SENDTO, args, 0, 0, 0, 0);
	if (ret != -1) {
		if (sent != NULL) {
			*sent = (size_t)ret;
		}
		return PLSTATUS_SUCCESS;
	}

	return ((errno == EAGAIN) ? PLSTATUS_TRY_AGAIN : PLSTATUS_FAILED);
}


