/*
 * Copyright 2007-2010 Claudio Castiglia
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/**
 * @file src/lib/plc/sysdep/linux/i386/plsys_gethostname.c
 */
#include <plc.h>


PLSTATUS PLEXPORT
plsys_uname(struct utsname *uname)
{
	if (uname == NULL) {
		return PLSTATUS_INVALID;
	}

	if (plsyscall(SYS_uname, &uname, 0, 0, 0, 0, 0) != -1) {
		return PLSTATUS_SUCCESS;
	}

	return ((errno == EAGAIN) ? PLSTATUS_TRY_AGAIN : PLSTATUS_FAILED);
}


PLSTATUS PLEXPORT
plsys_gethostname(char *name, size_t size)
{
	struct utsname uname;
	PLSTATUS st;
	char *p;

	if ((st = plsys_uname(&uname)) != PLSTATUS_SUCCESS) {
		return st;
	}

	for (p = uname.nodename; size && p != '\0'; size--) {
		*name++ = *p++;
	}

	*name = '\0';

	return PLSTATUS_SUCCESS;
}


