/*
 * Copyright 2007-2010 Claudio Castiglia
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/**
 * @file src/lib/plc/sysdep/linux/i386/plsys_sigaction.c
 */
#include <plc.h>


extern void *plsys_sigaction_restorer;
extern void *plsys_sigaction_restorer_rt;


PLSTATUS PLEXPORT
plsys_sigaction(int signo, const struct sigaction *act, struct sigaction *oact)
{
	struct kernel_sigaction kact, koact, *pkact;

	if (act != NULL) {
		kact.k_sa_handler = act->sa_handler;
		kact.sa_flags = act->sa_flags;
		kact.sa_restorer = act->sa_restorer;
		plsys_memmove(&kact.sa_mask, &act->sa_mask, sizeof(sigset_t));
		if (!(kact.sa_flags & SA_RESTORER)) {
			kact.sa_flags |= SA_RESTORER;
			kact.sa_restorer = (signo <= SIGUNUSED) ?
			    plsys_sigaction_restorer :
			    plsys_sigaction_restorer_rt;
		}
		pkact = &kact;
	} else {
		pkact = NULL;
	}

	if (plsyscall(SYS_sigaction, signo, pkact, &koact, 0, 0, 0) != -1) {
		if (oact != NULL) {
			oact->sa_handler = koact.k_sa_handler;
			oact->sa_flags = koact.sa_flags;
			oact->sa_restorer = koact.sa_restorer;
			plsys_memmove(&oact->sa_mask, &koact.sa_mask,
			    sizeof(sigset_t));
		}

		return PLSTATUS_SUCCESS;
	}

	return ((errno == EAGAIN) ? PLSTATUS_TRY_AGAIN : PLSTATUS_FAILED);
}


