/*
 * Copyright 2007-2010 Claudio Castiglia
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
#include <plc.h>

#include <dirent.h>
#include <stdio.h>
#include <string.h>


static void pldir(const char *path)
{
	struct dirent *de;
	long handle;
	PLSTATUS st;

	st = plsys_opendir(&handle, path);
	if (st != PLSTATUS_SUCCESS) {
		plprint("Can't plsys_opendir: %s", plstrstatus(st));
		return;
	}

	while (1) {
		st = plsys_readdir(handle, &de);
		if (st == PLSTATUS_SUCCESS) {
			plprint("\t%s\n", de->d_name);

		} else if (st == PLSTATUS_EOF) {
			break;

		} else {
			plprint("Can't plsys_readdir: %s - errno = %d\n",
			    plstrstatus(st), errno);
			break;

		}
	}

	plsys_closedir(handle);
}


static void dir(const char *path)
{
	struct dirent *de;
	DIR *dir;

	dir = opendir(path);
	if (dir == NULL) {
		fprintf(stderr, "Can't opendir: %s", strerror(errno));
		return;
	}

	while ((de = readdir(dir)) != NULL) {
		fprintf(stdout, "\t%s\n", de->d_name);
	}

	closedir(dir);
}


int
main(int argc, char **argv)
{
//	plinit();

	if (argc != 2) {
		plprint("pldir <dir>\n");
		return -1;
	}

	// peludo api
	if (0) 
		pldir(argv[1]);

	// posix api
	if (1)
		dir(argv[1]);

//	plrelease();
	return 0;
}


