/*
 * Copyright 2007-2010 Claudio Castiglia
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/*
 * Test of peludo poll() using posix wrappers.
 */
#include <sys/types.h>
#include <sys/socket.h> 
#include <netinet/in.h>
#include <poll.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

void test1();

int
main(int argc, char **argv)
{
	plprint("%s: Initializing PLC API...\n", __FUNCTION__);
//	plinit();

	plprint("%s: Running test...\n", __FUNCTION__);
	test1();

	plprint("%s: Releasing PLC API...\n", __FUNCTION__);
//	plrelease();

	return 0;
}

void
test1()
{
	int socket_desc;
	struct sockaddr_in address;
	int addrlen;
	int new_socket;
	char buf[1024];
	ssize_t len;
	int res, avail;
	struct pollfd pfd;

	socket_desc = socket(AF_INET, SOCK_STREAM, 0);
	if (socket_desc == 0) {
		perror("Can't create socket");
		exit(1);
	}

	printf("Socket created: %d\n", socket_desc);

	address.sin_family = AF_INET;
	address.sin_addr.s_addr = INADDR_ANY;
	address.sin_port = htons(1234);

	if (bind(socket_desc, (struct sockaddr *)&address, sizeof(address))
	    < 0) {
		perror("Can't bind");
		exit(1);
	}

	printf("Bind success\n");

	if (listen(socket_desc, 3) < 0) {
		perror("Can't listen");
		exit(1);
	}

	printf("Listen success\n");

	addrlen = sizeof(address);
	new_socket = accept(socket_desc, (struct sockaddr *)&address, &addrlen);
	if (new_socket < 0) {
		perror("Can't accept");
		exit(1);
	}

	printf("New socket is %d\n", new_socket);

#define WELCOME	"Peludo socket test1: Echo server started\n" \
		"Hi, write 'quit' to finish.\n"

	send(new_socket, WELCOME, strlen(WELCOME), 0);

	avail = 0;
	pfd.fd = new_socket;
	pfd.events = POLLIN | POLLPRI | POLLOUT | POLLERR | POLLHUP | POLLNVAL;
	while ((res = poll(&pfd, 1, -1)) > 0) {
		if (pfd.revents & (POLLIN | POLLPRI)) {
			len = recv(new_socket, buf, sizeof(buf), 0);
			if (len <= 0)  {
				perror("recv failed");
				break;
			}

			buf[len] = '\0';

			printf("READ: <%s>\n", buf);
			avail = 1;

			if (!strncmp(buf, "quit\n", len)) {
				break;
			}

		} else if (pfd.revents & POLLOUT) {
			if (!avail) {
				printf("POLLOUT but nothing to send\n");
			} else {
				send(new_socket, buf, len, 0);
				avail = 0;
			}

		} else {
			printf("revent: %x\n", pfd.revents);
		}
	}

	close(new_socket);
	close(socket_desc);
}


