#!/bin/sh
#
# Copyright 2007-2010 Claudio Castiglia
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
cat $@ | sort | awk '\
	BEGIN {
		total_time = 0
		total_count = 0
		total_symbols = 0
		last_symbol = ""
	}{
		total_count++
		total_time += $3
		if (last_symbol != $1) {
			last_symbol = $1
			symbol[total_symbols] = $1
			total_symbols++
		}
		count[$1]++
		elapsed[$1] += $3

	} END {
		printf("Calls Executed: %lu\n", total_count)
		printf("Symbols Executed: %lu\n", total_symbols)
		printf("Total time: %luus\n", total_time)

		printf("Summary:\n")

		for (i = 0; i < total_symbols; i++) {
			sym = symbol[i]
			printf("Symbol %s executed %lu times (%f%%), time consumed %luus (%f%%)\n",
			    sym, count[sym], 100 * count[sym] / total_count,
			    elapsed[sym], 100 * elapsed[sym] / total_time)
		}
	}'

