/*
 * Copyright 2007-2010 Claudio Castiglia
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/**
 * @file src/lib/plt/pthread_p.h
 * Peludo Threads - Private Prototypes.
 */
#ifndef PLTP_H
#define PLTP_H

#include <pldef.h>
#include <plspinlock.h>
#include <plist.h>


/**
 * States of a Thread.
 */
typedef enum _PTHREAD_STATE {
	PTHREAD_STATE_DEAD = 0,		/* Must be 0 */
	PTHREAD_STATE_RUNNING,
	PTHREAD_STATE_CANCELED
} PTHREAD_STATE;


/*
 * Condition flags handling.
 */
#define COND_FLAG_SIGNALED		(1 << 31)

#define COND_FLAG_MASK			(COND_FLAG_SIGNALED)

#define COND_IS_SIGNALED(cond)		((cond)->cond_value & \
					COND_FLAG_SIGNALED)

#define COND_CLEAR_FLAG(cond, flag)	((cond)->cond_value &= ~(flag))

#define COND_SET_FLAG(cond, flag)	((cond)->cond_value |= (flag))

#define COND_GET_COUNT(cond)		((cond)->cond_value & ~COND_FLAG_MASK)

#define COND_SET_COUNT(cond, val)	((cond)->cond_value =  \
					((cond)->cond_value & \
					COND_FLAG_MASK) | \
					(val & ~COND_FLAG_MASK))


/*
 * Private variables.
 */
extern pthread_t _pthread_main;
extern PLIST	 _pthread_list;
extern PLOCK	 _pthread_list_lock;


/*
 * Private prototypes.
 */
int		attr_stack_alloc(pthread_attr_t *);

PLSTATUS	tls_init();
void		tls_release();
void		tls_destructor(pthread_t);
void		pthread_check_cancelpoint();


#endif	/* PTHREAD_P_H */
