/*
 * Copyright 2007-2010 Claudio Castiglia
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/*
 * PLT API test of signals.
 */
#include <plc.h>

#include <sys/types.h>
#include <pthread.h>
#include <signal.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <unistd.h>

void test1();
void test2();
void test3();


int
main(int argc, char **argv)
{
#ifdef PLSTATIC
//	plinit();
	pthread_init_np();
#endif

	//test1();
	//test2();
	test3();

#ifdef PLSTATIC
	printf("%s: Releasing PTHREAD API...\n", __FUNCTION__);

	pthread_release_np();

	plprint("%s: Releasing PLC API...\n", __FUNCTION__);

	plrelease();
#endif

	return 0;
}


/*************************************************************
 * test-1:	IPC USING SIGNALS
 *************************************************************/
struct _test1 {
	volatile int newmsg;
	char	buffer[256];
} msg;

pthread_t thserver;
pthread_t thclient;

void *
test1_echo_client(void *arg)
{
	int signo, err;
	sigset_t set;

	printf("%s: Client Started...\n", __FUNCTION__);

	/*
	 * Block all signals.
	 */
	plsys_sigfillset(&set);
	err = pthread_sigmask(SIG_SETMASK, &set, NULL);
	if (err != 0) {
		printf("%s: pthread_sigmask1 failed, error=%d\n",
		    __FUNCTION__,
		    err);
		pthread_exit(NULL);
	}

	/*
	 * Main loop: Communicate with server thread using SIGUSR1.
	 */
	plsys_sigemptyset(&set);
	plsys_sigaddset(&set, SIGUSR1);

	while (1) {
		printf("%s: plt_sigwait()...\n", __FUNCTION__);

		err = sigwait(&set, &signo);
		if (err != 0) {
			printf("%s: sigwait() failed, error=%d\n",
			    __FUNCTION__,
			    err);
		} else {
			/*
			 * Signal received
			 */
			printf("%s: sigwait() returned because of "
			    "signal %d\n",
			    __FUNCTION__,
			    signo);

			printf("%s: Received message <%s>\n",
			    __FUNCTION__,
			    msg.buffer);

			printf("%s: Sending ack...\n", __FUNCTION__);

			err = pthread_kill(thserver, SIGUSR1);
			if (err != 0) {
				printf("%s: pthread_kill failed, error=%d\n",
				    __FUNCTION__,
				    err);
			}
		}
	}

	/* NOTREACHED */
	return NULL;
}


void *
test1_echo_server(void *arg)
{
	int i, signo, rsigno, err;
	sigset_t set;

	printf("%s: Server Started\n", __FUNCTION__);

	signo = SIGUSR1;
	i = 0;

	plsys_sigfillset(&set);
	err = pthread_sigmask(SIG_SETMASK, &set, NULL);
	if (err != 0) {
		printf("%s: pthread_sigmask failed, error=%d\n",
		    __FUNCTION__,
		    err);
	}

	plsys_sigemptyset(&set);
	plsys_sigaddset(&set, SIGUSR1);

	plsys_msleep(2000);

	while (1) {
		printf("%s: Writing new message %d...\n", __FUNCTION__, i);
		sprintf(msg.buffer, "message %d", i++);
		
		printf("%s: Sending signal %d to thread %p\n",
		    __FUNCTION__,
		    signo,
		    thclient);

		err = pthread_kill(thclient, signo);

		if (err != 0) {
			printf("%s: pthread_kill failed, error=%d\n",
			    __FUNCTION__,
			    err);
		} else {
			printf("%s: signal %d sent\n", __FUNCTION__, signo);
			printf("%s: Waiting the ack...\n", __FUNCTION__);

again:			printf("%s: sigwait()...\n", __FUNCTION__);

			err = sigwait(&set, &rsigno);
			if (err == 0) {
				printf("%s: received ack signal %d\n",
				    __FUNCTION__,
				    rsigno);

				continue;

			}

			printf("%s: ahora imprimo el errno\n", __FUNCTION__);
			printf("%s: sigwait ack failed error=%d\n",
			    __FUNCTION__,
			    err);

			goto again;
		}
	}

	/* NOTREACHED */
	return NULL;
}

void handler(int signo)
{
	printf("%s: signo=%d\n", __FUNCTION__, signo);
}

void
test1()
{
	struct sigaction act;
	PLSTATUS st;
	int err;

	bzero(&msg, sizeof(msg));

	printf("\n"
	    "/***********************************************************\n"
	    " * test-1:	IPC USING SIGNALS\n"
	    " ***********************************************************/\n");

	bzero(&act, sizeof(act));
	act.sa_handler = handler;
	plsys_sigfillset(&act.sa_mask);
	plsys_sigdelset(&act.sa_mask, SIGCANCEL);
	if ((st = plsys_sigaction(SIGUSR1, &act, NULL)) != PLSTATUS_SUCCESS) {
		printf("%s: Can't sigaction: %s, errno=%d\n",
		    __FUNCTION__,
		    plstrstatus(st),
		    errno);
		return;
	}

	/*
 	 * Create threads
 	 */
	printf("%s: Creating echo server...\n", __FUNCTION__);
	err = pthread_create(&thserver, NULL, test1_echo_server, NULL);
	if (err != 0) {
		printf("%s: Can't pthread_create server: error=%d\n",
		    __FUNCTION__,
		    err);
		return;
	}

	printf("%s: echo server created at %x...\n",
	    __FUNCTION__,
	    thserver);

	printf("%s: Creating echo client...\n", __FUNCTION__);
	err = pthread_create(&thclient, NULL, test1_echo_client, NULL);
	if (err != 0) {
		printf("%s: Can't pthread_create client: error=%d\n",
		    __FUNCTION__,
		    err);
		return;
	}

	printf("%s: echo client created at %x...\n",
	    __FUNCTION__,
	    thclient);

	plsys_msleep(5000);
}

/*************************************************************
 * test-2:	SIGNAL INHERITANCE
 *************************************************************/
pthread_cond_t end;
pthread_mutex_t endm;
pthread_t child2, child22, child3;

void
test2_handler(int signo)
{
	printf("Signal %d\n", signo);
}


void *
test2_child(void *arg)
{
	long n = (long)arg;

	printf("CHILD %d BEGIN\n", n);

	if (n == 1) {
		/*
		 * The 1st child spawns other 2 threads.
		 */
		pthread_create(&child2, NULL, test2_child, (void *)2);
		pthread_create(&child3, NULL, test2_child, (void *)3);

	} else if (n == 2) {
		pthread_create(&child22, NULL, test2_child, (void *)22);
		while (1) {
			printf("I am killing myself with SIGINT..\n");
			plsys_tkill(plsys_gettid(), SIGINT);
			plsys_msleep(100);
		}

	} else if (n == 22) {
		int t;

		/*
		 * The 2nd, child of the 1st, do something.
		 */
		for (t = 0;; t++) {
			printf("I am %d, count %d...\n", n, t);
			plsys_msleep(500);
		}

	} else {
		/*
		 * The 3rd, child of the 1st, send signals
		 */
		while (child22 == NULL) plsys_msleep(1000);
		plsys_msleep(1000);

		printf("CHILD 3: killing CHILD 22...\n");
		pthread_kill(child22, SIGUSR1);
		plsys_msleep(1000);
		printf("CHILD 3: killing CHILD 22...\n");
		pthread_kill(child22, SIGUSR1);

		plsys_msleep(5000);
		printf("CHILD 3: SENDING END CONDITION...\n");
		pthread_cond_signal(&end);
		pthread_exit((void *)n);
	}

	pthread_mutex_lock(&endm);
	pthread_cond_wait(&end, &endm);
	pthread_mutex_unlock(&endm);
	pthread_exit((void *)n);

	return NULL;
}


void
test2()
{
#if 1
	struct sigaction act;
	pthread_t t;
	PLSTATUS st;

	bzero(&act, sizeof(act));
	act.sa_handler = test2_handler;
	if ((st = plsys_sigaction(SIGUSR1, &act, NULL)) != PLSTATUS_SUCCESS) {
		printf("%s: Can't sigaction: %s, errno=%d\n",
		    __FUNCTION__,
		    plstrstatus(st),
		    errno);
		return;
	}

	pthread_mutex_init(&endm, NULL);
	pthread_cond_init(&end, NULL);

	pthread_mutex_lock(&endm);

	pthread_create(&t, NULL, test2_child, (void *)1);

	pthread_cond_wait(&end, &endm);

	printf("End condition signaled. finishing...\n");
#else

	printf("I am %d\n", plsys_gettid());
	plsys_tkill(plsys_gettid(), SIGINT);

#endif
}


/*************************************************************
 * test-3:	SIGSUSPEND
 *************************************************************/

int suspend = 0;

void *
thread3(void *arg)
{
	pid_t tid;
	int i;

	tid = gettid();

	for (i = 0;; i++) {
		printf("%s: %d, hola %d\n", __FUNCTION__, tid, i);
		plsys_msleep(50);
	}

	return NULL;
}

void
handler3(int signo)
{
    	sigset_t mask;

	//write(STDOUT_FILENO, "entro\n", sizeof("entro\n"));
	printf("\n%s: received signal %d\n", __FUNCTION__, signo);
	sigfillset(&mask);
	sigdelset(&mask, SIGUSR1);
	sigdelset(&mask, SIGTERM);

	while (suspend) {
		sigsuspend(&mask);
	}

#ifndef PLSTATIC
/*
	asm("leave\n");
	asm("pop %eax \n");
	asm("mov %eax, 0x77 \n");
	asm("int $0x80 \n");
*/
#endif
}

void
test3()
{
	struct sigaction act;
	sigset_t mask;
	pthread_t t;

	printf("%s: Setting signal handler...\n", __FUNCTION__);
	act.sa_handler = handler3;
	sigemptyset(&act.sa_mask);
	sigaction(SIGUSR1, &act, NULL);

	printf("%s: Blocking undesired signals...\n", __FUNCTION__);
	sigemptyset(&mask);
	sigaddset(&mask, SIGQUIT);
	sigaddset(&mask, SIGINT);
	sigaddset(&mask, SIGPIPE);
	sigprocmask(SIG_BLOCK, &mask, NULL);

	printf("%s: Creating child thread...\n", __FUNCTION__);
	if (pthread_create(&t, NULL, thread3, NULL)) {
		printf("Can't pthread_create: %s", strerror(errno));
		exit(-1);
	}

	plsys_msleep(1000);
	suspend = 1;
	printf("%s: Suspending thread...\n", __FUNCTION__);
	pthread_kill(t, SIGUSR1);
	printf("%s: killed...\n", __FUNCTION__);
	plsys_msleep(2000);
	printf("%s: Killing with USR1...\n", __FUNCTION__);
	pthread_kill(t, SIGUSR1);
	printf("%s: Unsuspending thread...\n", __FUNCTION__);
	suspend = 0;
	pthread_kill(t, SIGUSR1);

	plsys_msleep(2000);

	exit(0);
}


