/*
 * Copyright 2007-2010 Claudio Castiglia
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/**
 * @file src/tools/plzma/plzma.c
 * Peluded lzma encoder/decoder.
 */
#include <plc.h>
#include <plzma.h>

#include <sys/types.h>
#include <sys/stat.h>

#include <errno.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

static char *progname;


int
main(int argc, char **argv)
{
	int in, out, encode;
	struct stat stat;
	void *dst, *src;
	size_t dstsiz;
	PLSTATUS st;

	dst = NULL; //shut-up gcc

//	plinit();
	plzma_init();

	progname = argv[0];

	if (argc != 4) {
		printf("usage: %s e|d <infile> <outfile>\n", progname);
		exit(-1);
		/* NOTREACHED */
	}

	encode = (*argv[1] == 'e' || *argv[1] == 'E');

	in = open(argv[2], O_RDONLY, 0);
	if (in < 0) {
		printf("%s: Can't open %s: %s\n", progname, argv[2],
		    strerror(errno));
		exit(-1);
		/* NOTREACHED */
	}

	if (fstat(in, &stat) < 0) {
		printf("%s: Can't stat %s: %s\n", progname, argv[2],
		    strerror(errno));
		exit(-1);
	}

	if (stat.st_size == 0) {
		printf("%s: Input size is 0: %s\n", progname, argv[2]);
		exit(-1);
	}

	src = malloc(stat.st_size);
	if (src == NULL) {
		printf("%s: Can't malloc: %s\n", progname, strerror(errno));
		exit(-1);
	}

	if (read(in, src, stat.st_size) < 0) {
		printf("%s: Can't read: %s: %s\n", progname, argv[2],
		    strerror(errno));
		exit(-1);
	}

	out = open(argv[3], O_CREAT | O_TRUNC | O_RDWR, 0644);
	if (out < 0) {
		printf("%s: Can't create %s: %s\n", progname, argv[2],
		    strerror(errno));
		exit(-1);
		/* NOTREACHED */
	}

	if (encode) {
		st = plzma_encode((long *)&dst, &dstsiz, (long)src,
		    stat.st_size);
		if (st != PLSTATUS_SUCCESS) {
			printf("%s: Can't encode: %d \"%s\"\n",
			    progname,
			    st,
			    plstrstatus(st));

			exit(-1);
		}
	} else {
		st = plzma_decode((long *)&dst, &dstsiz, (long)src,
		    stat.st_size);

		if (st != PLSTATUS_SUCCESS) {
			printf("%s: Can't decode: %d \"%s\"\n",
			    progname,
			    st,
			    plstrstatus(st));

			exit(-1);
		}
	}

	if (write(out, dst, dstsiz) != dstsiz) {
		printf("%s: Can't write: %s: %s\n", progname, argv[3],
		    strerror(errno));
		exit(-1);
	}

	close(in);
	close(out);

	return 0;
}


