/*
 * Copyright 2007-2010 Claudio Castiglia
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/**
 * @file src/tools/prun/prun.c
 * Launch a PLB File.
 */
#include <plc.h>
#include <string.h>


static int
create_args(char *dst, ssize_t dstsiz, int argc, char **argv)
{
	size_t siz;
	int i;

	for (i = 0; i < argc && dstsiz > 0; i++) {
		siz = strlcpy(dst, argv[i], dstsiz) + 1;
		dst += siz;
		dstsiz -= siz;
	}

	/* Finish the string list */
	if (dstsiz) {
		*dst = '\0';
	}

	return (dstsiz > 0);
}


int
main(int argc, char **argv)
{
	char argstr[PLPAGE_SIZE];
	int (*boot)(long);
	int exitcode;
	stat_t st;
	long fd;

//	plinit();

	if (argc < 2) {
		plprint("usage: %s <peludo>\n", argv[0]);
		plsys_exit(-1);
		/* NOTREACHED */
	}

	if (plsys_open(&fd, argv[1], O_RDONLY, 0) != PLSTATUS_SUCCESS) {
		plprint("%s: Can't open: errno=%d\n", argv[0], errno);
		plsys_exit(-1);
		/* NOTREACHED */
	}

	if (plsys_fstat(fd, &st) != PLSTATUS_SUCCESS) {
		plprint(" %s: Can't stat: errno=%d\n", argv[1], errno);
		plsys_exit(-1);
		/* NOTREACHED */
	}

	if (plsys_mmapfile((long *)&boot, fd, st.st_size, MAP_PRIVATE,
	    PROT_READ | PROT_EXEC, 0) != PLSTATUS_SUCCESS) {
		plprint( "%s: Can't mmapfile: errno=%d\n", argv[1], errno);
		plsys_exit(-1);
		/* NOTREACHED */
	}

	if (!create_args(argstr, sizeof(argstr), argc - 1, argv + 1)) {
		plprint("%s: Too many arguments\n", argv[0]);
		plsys_exit(-1);
		/* NOTREACHED */
	}

	exitcode = boot((long)argstr);

	(void)plsys_munmap((long)boot, st.st_size);
	plsys_close(fd);

	plprint("\n%s: Peludo exited with code: %d\n", argv[0], exitcode);

//	plrelease();

	return exitcode;
}

